﻿/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;


namespace ConsoleApplication1
{
    class DataSourceSample
    {
        private MailingService service;
        private SampleHelper helper;

        public DataSourceSample(MailingService service)
        {
            this.service = service;
            helper = new SampleHelper(service);
        }


        /* Create an Internal Data Source with 3 columns: id, email_address, and name.
         *
         * dsname is the name of data source.
         * return the ID of created internal data source.
         */
        public InternalDataSourceId createIDS(string dsname)
        {
            // Create Internal Data Source and fill in properties.
            InternalDataSource dataSource = new InternalDataSource();

            dataSource.description = "Internal Data Source from .NET";
            dataSource.name = dsname;

            // Specify data source fields: id (primary key), email and name.
            // Server will generate "unsub" and "email_status" fields
            DataSourceField[] fields = new DataSourceField[3];

            DataSourceField primaryKey = new DataSourceField();
            primaryKey.dataType = DataSourceDataType.DATA;
            primaryKey.fieldType = DataSourceFieldType.INTEGER;
            primaryKey.isPrimaryKey = true;
            primaryKey.name = "id";
            primaryKey.writebackEnabled = false;
            fields[0] = primaryKey;

            DataSourceField email = new DataSourceField();
            email.dataType = DataSourceDataType.DATA;
            email.fieldType = DataSourceFieldType.EMAIL;
            email.isPrimaryKey = false;
            email.name = "email_address";
            email.writebackEnabled = false;
            fields[1] = email;

            DataSourceField name = new DataSourceField();
            name.dataType = DataSourceDataType.DATA;
            name.fieldType = DataSourceFieldType.TEXT;
            name.isPrimaryKey = false;
            name.name = "name";
            name.writebackEnabled = false;
            fields[2] = name;

            // Assign fields to data source
            dataSource.field = fields;

            // Create data source
            InternalDataSourceId dataSourceId = (InternalDataSourceId) helper.create(dataSource);

            // Add some data
            addRecordsInline(dataSourceId);

            return dataSourceId;
        }


        /* Create an External Data Source: Local copy
         *
         * edsName is the name of External Data Source
         * return the EDS ID that is created
         */
        public ExternalDataSourceId createEDS(string edsName)
        {
            ExternalDataSource eds = new ExternalDataSource();

            // Set name, description, and local copy to true.
            eds.name = edsName;
            eds.description = "External Data Source from .NET";

            // Set connection info
            ExternalDataSourceConnectionInfo info = new ExternalDataSourceConnectionInfo();
            info.databaseName = "DATABASE_NAME";
            info.databaseType = DatabaseType.ORACLE;
            info.hostname = "10.0.0.0";
            info.password = "PASSWORD";
            info.port = "1521";
            info.username = "USERNAME";
            eds.connectionInfo = info;

            // Make it a local copy
            eds.enableLocalCopy = true;

            // Set elements of choice type: the order is important.
            // Items and ItemsElementName should match
            ExternalDataSourceDailyRefresh dailyRefresh = new ExternalDataSourceDailyRefresh();
            dailyRefresh.startTime = new DateTime();
            eds.Items = new Object[]{true, dailyRefresh, "WRITEBACK_TABLE", "SOURCE_TABLE"};
            eds.ItemsElementName = new ItemsChoiceType[] {
                ItemsChoiceType.allowRefreshAtLaunchTime,
                ItemsChoiceType.dailyRefresh,
                ItemsChoiceType.writebackTable,
                ItemsChoiceType.sourceTableName};

            // Set fields: need to make sure the fields match EDS columns
            DataSourceField[] fields = new DataSourceField[7];

            DataSourceField primaryKey = new DataSourceField();
            primaryKey.dataType = DataSourceDataType.DATA;
            primaryKey.fieldType = DataSourceFieldType.INTEGER;
            primaryKey.isPrimaryKey = true;
            primaryKey.name = "id";
            primaryKey.writebackEnabled = false;
            fields[0] = primaryKey;

            DataSourceField email = new DataSourceField();
            email.dataType = DataSourceDataType.DATA;
            email.fieldType = DataSourceFieldType.EMAIL;
            email.isPrimaryKey = false;
            email.name = "email";
            email.writebackEnabled = false;
            fields[1] = email;

            DataSourceField phone = new DataSourceField();
            phone.dataType = DataSourceDataType.DATA;
            phone.fieldType = DataSourceFieldType.SMS;
            phone.isPrimaryKey = false;
            phone.name = "phone";
            phone.writebackEnabled = false;
            fields[2] = phone;

            DataSourceField status = new DataSourceField();
            status.dataType = DataSourceDataType.STATUS;
            status.fieldType = DataSourceFieldType.TEXT;
            status.isPrimaryKey = false;
            status.name = "email_status";
            status.writebackEnabled = false;
            fields[3] = status;

            DataSourceField unsub = new DataSourceField();
            unsub.dataType = DataSourceDataType.UNSUBSCRIBE;
            unsub.fieldType = DataSourceFieldType.TEXT;
            unsub.isPrimaryKey = false;
            unsub.name = "unsub";
            unsub.writebackEnabled = false;
            fields[4] = unsub;

            DataSourceField fname = new DataSourceField();
            fname.dataType = DataSourceDataType.DATA;
            fname.fieldType = DataSourceFieldType.TEXT;
            fname.isPrimaryKey = false;
            fname.name = "first_name";
            fname.writebackEnabled = false;
            fields[5] = fname;

            DataSourceField lname = new DataSourceField();
            lname.dataType = DataSourceDataType.DATA;
            lname.fieldType = DataSourceFieldType.TEXT;
            lname.isPrimaryKey = false;
            lname.name = "last_name";
            lname.writebackEnabled = true;
            fields[6] = lname;

            eds.field = fields;

            return (ExternalDataSourceId) helper.create(eds);
        }


        /* Add one record using inline to the data source created in createIDS().
         *
         * dataSourceId is the data source ID where record will be added.
         */
        public void addRecordsInline(DataSourceId dataSourceId)
        {
            // Create request
            AddDataSourceRecordsRequest request = new AddDataSourceRecordsRequest();
            request.dataSourceId = dataSourceId;

            // Create record based on the data source schema created in createIDS()
            DataSourceRecord[] records = new DataSourceRecord[1];
            DataSourceRecord record_1 = new DataSourceRecord();

            NameValuePair[] values = new NameValuePair[3];
            NameValuePair idCol = new NameValuePair();
            idCol.name = "id";
            idCol.value = "1";
            values[1] = idCol;

            NameValuePair emailCol = new NameValuePair();
            emailCol.name = "email_address";
            emailCol.value = "email_for_dotnet@dotnet.com";
            values[0] = emailCol;

            NameValuePair nameCol = new NameValuePair();
            nameCol.name = "name";
            nameCol.value = "dotnet_name";
            values[2] = nameCol;

            // Add record to request and make call
            record_1.field = values;
            records[0] = record_1;
            request.Items = records;

            service.addRecords(new addRecords(request));
        }

        /*
         * Upsert the record of the given datasource.
         */
        public void upsertRecord(DataSourceId dataSourceId)
        {
            // Upsert datasource request
            UpsertDataSourceRecordsRequest request = new UpsertDataSourceRecordsRequest();

            request.dataSourceId = dataSourceId;
            // Set datasource record to be updated createIDS()
            NameValuePair[] values = new NameValuePair[3];
            NameValuePair emailCol = new NameValuePair();
            emailCol.name = "email_address";
            emailCol.value = "email_for_dotnet_upserted@dotnet.com";
            values[0] = emailCol;

            NameValuePair idCol = new NameValuePair();
            idCol.name = "id";
            idCol.value = "1";
            values[1] = idCol;

            NameValuePair nameCol = new NameValuePair();
            nameCol.name = "name";
            nameCol.value = "dotnet_name_upserted";
            values[2] = nameCol;

            DataSourceRecord dataSourceRecord = new DataSourceRecord();
            dataSourceRecord.field = values;

            request.dataSourceRecord = new DataSourceRecord[] { dataSourceRecord };

            // For this api we do not need to wait for IDLE state of datasource.
            service.upsertRecord(new upsertRecords(request));
        }

        /*
        * GetRecords of the record of the given datasource.
        */
        public void getRecords(DataSourceId dataSourceId)
        {
            // Upsert datasource request
            GetDataSourceRecordsRequest request = new GetDataSourceRecordsRequest();

            request.dataSourceId = dataSourceId;
            // Set datasource record to be updated createIDS()
            NameValuePair idCond = new NameValuePair();
            idCond.name = "id";
            idCond.value = "1";

            request.matchFields = new NameValuePair[] { idCond };

            // For this api we do not need to wait for IDLE state of datasource.
            GetRecordsResponse response = service.getRecords(new getRecords(request)).getRecordsResponse;
            DataSourceRecord[] records = ((GetDataSourceRecordsResponse)response).dataSourceRecord;
            Console.WriteLine(" Matching records found " + records.Length);
        }


        /* Add records using a file to the data source created in createIDS().
         *
         * dataSourceId is the data source ID where records will be added.
         * dataFile is the file that will be uploaded.
         */
        public void addRecordsUsingFile(DataSourceId dataSourceId, FileStream dataFile)
        {
            // Create request
            AddDataSourceRecordsRequest request = new AddDataSourceRecordsRequest();
            request.dataSourceId = dataSourceId;
            request.containsFieldNames = true;
            request.containsFieldNamesSpecified = true;
            request.fieldDelimiter = ",";

            // Read file and add to request
            //FileStream recordFile = File.OpenRead(DATASOURCE_RECORD_FILE);
            BinaryReader br = new BinaryReader(dataFile);
            Object[] objs = new Object[1];
            objs[0] = br.ReadBytes((int)dataFile.Length);
            request.Items = objs;

            service.addRecords(new addRecords(request));
        }


        /* Remove the record added in addRecordsInline().
         *
         * dataSourceId is the data source ID whose records will be removed.
         * return the number of records that was removed.
         */
        public int removeRecordsInline(DataSourceId dataSourceId)
        {
            // Create request and remove by matching name column
            RemoveDataSourceRecordsRequest request = new RemoveDataSourceRecordsRequest();
            request.dataSourceId = dataSourceId;
            request.matchFieldName = "name";

            // Name column will match these values
            object[] records = new object[1];
            records[0] = "dotnet_name";

            // Set request and make call
            request.Items = records;
            return service.removeRecords(new removeRecords(request)).removeRecordsResponse.recordsRemoved;
        }

         /* Removes duplicate records of specified column names */
         public bool dedupRecords(DataSourceId dsId)
         {
            DedupeDataSourceRecordsRequest request = new DedupeDataSourceRecordsRequest();
            if (dsId is InternalDataSourceId || dsId is ExtensionId)
            {
               request.Item = dsId;
            }
            else
            {
              throw new Exception("Unsupported operation");
            }

                request.matchField = new string[] { "email1" };
                //request.option = "KEEP_NEWEST";
                //return service.dedupeRecords(new dedupeRecords(request)).dedupeRecordsResponse.success;
                return false;
            }
        /* Export a data source records to a file.
         *
         * dataSourceId is the data source to be exported.
         * toFile is the file to be written to.
         */
        public void exportRecords(DataSourceId dataSourceId, FileStream toFile)
        {
            // Create and fill in request
            ExportDataSourceRecordsRequest request = new ExportDataSourceRecordsRequest();
            request.dataSourceId = dataSourceId;
            request.fieldDelimiter = "|";
            request.rowDelimiter = "\n\r";

            // Export regular records. Use true to export malformed records.
            request.useMalformedRecords = false;

            // Wait for data source status to be idle before exporting
            helper.waitForIdle(dataSourceId);

            // Make call and save data to file
            ExportRecordsResponse response = service.exportRecords(new exportRecords(request)).exportRecordsResponse;
            toFile.Write(response.data, 0, response.data.Length);
        }


        /* Retrieve and print out an data source statistics.
         *
         * dataSourceId is the data source ID that will be retrieved and printed.
         */
        public void getAndPrintStatistics(DataSourceId dataSourceId)
        {
            // Create request
            GetDataSourceStatisticsRequest request = new GetDataSourceStatisticsRequest();
            request.dataSourceId = dataSourceId;

            // Make call
            GetDataSourceStatisticsResponse response = (GetDataSourceStatisticsResponse) service.getStatistics(new getStatistics(request)).getStatisticsResponse;

            // Print out statistics
            Console.WriteLine("\nStatistics for data source " + dataSourceId.id);
            Console.WriteLine("Total records: " + response.totalRecords);
            Console.WriteLine("Total invalid records: " + response.totalInvalidRecords);
            Console.WriteLine("Total malformed records: " + response.totalMalformedRecords);
            Console.WriteLine("Total unsub records: " + response.totalUnsubscribedRecords);
        }


        /* Get and print data source content to console.
         *
         * dataSourceId is the data source ID that will be retrieved and printed
         */
        public void getAndPrint(DataSourceId dataSourceId)
        {
            DataSource dataSource = (DataSource) helper.get(dataSourceId);

            // Print data source to console
            Console.WriteLine("\nDataSource:");
            Console.WriteLine("\tID: " + dataSource.objectId.id);
            Console.WriteLine("\tName: " + dataSource.name);
            Console.WriteLine("\tDescription: " + dataSource.description);
            Console.WriteLine("\tCampaignId: " + dataSource.campaignId);
            Console.WriteLine("\tCreated Time: " + dataSource.createdTime);
            Console.WriteLine("\tModified Time: " + dataSource.modifiedTime);
            Console.WriteLine("\tOperationStatus: " + dataSource.operationStatus);
            Console.WriteLine("\tOrganization: " + dataSource.organizationId.id);

            // Print out field definition (data source schema)
            Console.WriteLine("\tUserId: " + dataSource.ownerId.id);
            Console.WriteLine("\tFields: ");
            for (int i = 0; i < dataSource.field.Length; i++ )
            {
                Console.WriteLine("\t\t" + dataSource.field[i].name + " " + dataSource.field[i].fieldType);
            }

            if (dataSourceId is ExternalDataSourceId)
            {
                ExternalDataSource eds = (ExternalDataSource)dataSource;
                Console.WriteLine("\tItems Size: " + eds.Items.Length);
                Console.WriteLine("\tLocalCopy: " + eds.enableLocalCopy);
                Console.WriteLine("\tConnection: " +
                    eds.connectionInfo.databaseName + " " +
                    eds.connectionInfo.databaseType + " " +
                    eds.connectionInfo.hostname + " " +
                    eds.connectionInfo.port + " " +
                    eds.connectionInfo.username);
            }
        }


        /* Updatea a data source with new name.
         *
         * dataSourceId is the data source ID whose name will be updated.
         * newName is the new name for the data source.
         */
        public void updateDataSourceName(DataSourceId dataSourceId, String newName)
        {
            // Get the data source first
            DataSource dataSource = (DataSource) helper.get(dataSourceId);
            dataSource.name = newName;

            // Update the data source with new name
            helper.update(dataSource);
        }


        /* Copy a data source using a new name.
         *
         * dataSourceId is the data source ID that will be copied.
         * newName is the name of new data source.
         * return the new data source ID.
         */
        public InternalDataSourceId copyDataSource(InternalDataSourceId dataSourceId, String newName)
        {
            // Create copy request
            CopyDataSourceRequest request = new CopyDataSourceRequest();
            request.Item = dataSourceId;
            request.newName = newName;

            return (InternalDataSourceId) service.copy(new copy(request)).copyResponse.objectId;
        }


        /* Refresh External Data Source records.
         *
         * edsId is the EDS ID to be refreshed.
         * return if refresh is successful.
         */
        public Boolean refreshRecords(ExternalDataSourceId edsId)
        {
            RefreshRecordsRequest request = new RefreshRecordsRequest();
            request.dataSourceId = edsId;
            return service.refreshRecords(new refreshRecords(request)).refreshRecordsResponse.success;
        }


        /* Cancel refreshing External Data Source records.
         *
         * edsId is the EDS ID to be refreshed.
         * return if cancelling is successful.
         */
        public Boolean cancelRefreshRecords(ExternalDataSourceId edsId)
        {
            CancelRefreshRecordsRequest request = new CancelRefreshRecordsRequest();
            request.dataSourceId = edsId;
            return service.cancelRefreshRecords(new cancelRefreshRecords(request)).cancelRefreshRecordsResponse.success;
        }


        /* List internal data source by a name.
         * It will return zero or one data source for an organization.
         *
         * name is the data source name that will be listed
         * return the internal data source ID that matches the give name.
         */
        public InternalDataSourceId listInternalDataSourcesByName(string name)
        {
            // Create list request and filter
            ListRequest request = new ListRequest();
            DataSourceFilter filter = new DataSourceFilter();

            // Set name condition
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = name;

            filter.nameCondition = nameCondition;
            setAttributeAndTypeToFilter(filter);
            request.filter = filter;

            return (InternalDataSourceId) service.list(new list(request)).listResponse.objectId[0];
        }


        /* List internal data source belonging to a user.
         *
         */
        public void listInternalDataSourcesByUser(UserId userId)
        {
            // Creae list request and filter
            ListRequest request = new ListRequest();
            DataSourceFilter filter = new DataSourceFilter();

            // Set user condition
            ScalarIdFilterCondition userCondition = new ScalarIdFilterCondition();
            userCondition.@operator = FilterIdScalarOperator.EQUAL;
            userCondition.value = userId;
            filter.userIdCondition = userCondition;

            setAttributeAndTypeToFilter(filter);
            request.filter = filter;

            ObjectId[] dataSourceIds = service.list(new list(request)).listResponse.objectId;

            // Print out all IDs
            Console.WriteLine("\nGot the following IDSs for User" + userId.id + ": ");
            for (int i = 0; i < dataSourceIds.Length; i++)
            {
                Console.Write(dataSourceIds[i].id + ", ");
            }
        }




        /* Set filter type to internal.
         */
        private void setAttributeAndTypeToFilter(DataSourceFilter filter)
        {
            // Set type condition: required for data source
            ScalarStringFilterCondition typeCondition = new ScalarStringFilterCondition();
            typeCondition.@operator = FilterStringScalarOperator.EQUAL;
            typeCondition.value = "INTERNAL"; // See schema. .NET seems ignore DataSourceType element.
            filter.typeCondition = typeCondition;

            helper.setFilter(filter);
        }
    }
}
