﻿/* Copyright 2014, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ConsoleApplication1
{
    class MailingSample
    {
        private MailingService service;
        private SampleHelper helper;
        private SystemAddressId bounceId;
        private SystemAddressId fromId;
        private SystemAddressId replyId;

        public MailingSample(MailingService service, SystemAddressId bounceId, SystemAddressId fromId, SystemAddressId replyId)
        {
            this.service = service;
            this.bounceId = bounceId;
            this.fromId = fromId;
            this.replyId = replyId;
            this.helper = new SampleHelper(service);
        }


        /* Create a one_time mailing with no excluded target. Use only one of instead of list of
         * target, seed list, suppresion list, attachment and content blocks.
         * 
         * mailingName is the name of the mailing.
         * targetId is the included target ID used for the mailing.
         * seedListId is the seed list ID used with this mailing.
         * suppressionListId is the suppresion list ID used with this mailing.
         * attachmentId is the attachment ID used with this mailing.
         * contentBlockId is the content block ID used with this mailing.
         * templateId is the template ID used with this mailing.
         * 
         * return the ID of created standard mailing.
         */ 
        public StandardMailingId createOneTimeStandardMailing(
            String mailingName, 
            TargetId targetId, 
            SeedListId seedListId, 
            SuppressionListId suppressionListId,
            AttachmentId attachmentId, 
            ContentBlockId contentBlockId, 
            TemplateId templateId)
        {
            StandardMailing mailing = new StandardMailing();
            setMailingCommonFields(mailing, mailingName, bounceId, fromId, replyId, templateId);

            mailing.description = "StandardMailing from dotnet";
            mailing.eliminateDuplicates = true;
            mailing.type = MailingType.ONE_TIME;
            mailing.typeSpecified = true;
            
            SchedulableMailingSchedule schedule = new SchedulableMailingSchedule();
            schedule.startDateTime = new DateTime(2020, 01, 01);
            
            mailing.schedule = schedule;

            // Set attachment
            mailing.attachmentId = new AttachmentId[] {attachmentId};
            mailing.contentBlockId = new ContentBlockId[] {contentBlockId} ;
            mailing.includedTargetId = new TargetId[] {targetId};
            mailing.seedListId = new SeedListId[] {seedListId};
            mailing.suppressionListId = new SuppressionListId[] {suppressionListId};

            return (StandardMailingId) helper.create(mailing);
        }

/* Create a recurring mailing with no excluded target. Use only one of instead of list of
         * target, seed list, suppresion list, attachment and content blocks.
         *
         * mailingName is the name of the mailing.
         * targetId is the included target ID used for the mailing.
         * seedListId is the seed list ID used with this mailing.
         * suppressionListId is the suppresion list ID used with this mailing.
         * attachmentId is the attachment ID used with this mailing.
         * contentBlockId is the content block ID used with this mailing.
         * templateId is the template ID used with this mailing.
         *
         * return the ID of created standard mailing.
         */
        public StandardMailingId createStandardMailing(
            String mailingName,
            TargetId targetId,
            SeedListId seedListId,
            SuppressionListId suppressionListId,
            AttachmentId attachmentId,
            ContentBlockId contentBlockId,
            TemplateId templateId)
        {
            StandardMailing mailing = new StandardMailing();
            setMailingCommonFields(mailing, mailingName, bounceId, fromId, replyId, templateId);

            mailing.description = "StandardMailing from dotnet";
            mailing.eliminateDuplicates = true;
            mailing.type = MailingType.RECURRING;
            mailing.typeSpecified = true;

            // Set schedule: daily recurrence with every two days
            SchedulableMailingScheduleRecurrenceDailyRecurrence daily = new SchedulableMailingScheduleRecurrenceDailyRecurrence();
            daily.Item = 2; // every two days

            SchedulableMailingScheduleRecurrence recurrence = new SchedulableMailingScheduleRecurrence();
            recurrence.Item1 = daily;
            recurrence.Item = new DateTime(2020, 10, 10);

            SchedulableMailingSchedule schedule = new SchedulableMailingSchedule();
            schedule.startDateTime = new DateTime(2020, 01, 01);

            schedule.recurrence = recurrence;
            mailing.schedule = schedule;

            // Set attachment
            mailing.attachmentId = new AttachmentId[] {attachmentId};
            mailing.contentBlockId = new ContentBlockId[] {contentBlockId} ;
            mailing.includedTargetId = new TargetId[] {targetId};
            mailing.seedListId = new SeedListId[] {seedListId};
            mailing.suppressionListId = new SuppressionListId[] {suppressionListId};

            return (StandardMailingId) helper.create(mailing);
        }


        /* Create a transactional mailing.
         * 
         * mailingName is the name of the mailing.
         * targetId is the target used for the mailing.
         * templateId is the template used for the mailing.
         * contentBlockId is the content block used for the mailing.
         * templateId is the template used in the mailing.
         * 
         * return the ID of created transactional mailing.
         */
        public TransactionalMailingId createTransactionalMailing(
            string mailingName, 
            TargetId targetId, 
            TemplateId templateId)
        {
            TransactionalMailing txMailing = new TransactionalMailing();
            setMailingCommonFields(txMailing, mailingName, bounceId, fromId, replyId, templateId);

            txMailing.description = "TransactionalMailing from dotnet";
            txMailing.type = MailingType.TRANSACTIONAL;
            txMailing.typeSpecified = true;
            txMailing.messageType = MessageType.EMAIL;
            txMailing.targetId = targetId;

            return (TransactionalMailingId) helper.create(txMailing);
        }


        /* Set mailing properties common to both standard and transactional mailings.
         * 
         */
        private void setMailingCommonFields(
            Mailing mailing, 
            String mailingName, 
            SystemAddressId bounceId, 
            SystemAddressId fromId, 
            SystemAddressId replyId, 
            TemplateId templateId)
        {
            mailing.bodyEncoding = Encoding.BASE64;
            mailing.bounceAddressId = bounceId;
            mailing.fieldDelimiter = "::";
            mailing.fromAddressId = fromId;
            mailing.fromName = "From Name";
            mailing.headerEncoding = Encoding.EIGHT_BIT;
            mailing.isApproved = true;
            mailing.isCompliant = true;
            mailing.name = mailingName;

            // Use outputBodyCharSet(Token)
            mailing.Item = CharSet.GB2312;

            // Use outputHeaderCharSet(Token)
            mailing.Item1 = CharSet.GB2312;

            mailing.priority = MailingPriority.NORMAL;
            mailing.replyAddressId = replyId;
            mailing.rowDelimiter = "\n";
            mailing.subject = "mailing subject";
            mailing.templateId = templateId;
            mailing.format = new MessageFormat[] {MessageFormat.HTML};
            mailing.header = new String[] {"mailing header"};
        }


        /* Get and print out a mailing.
         * 
         * mailingId is the ID of the mailing to be printed.
         */
        public void getAndPrint(MailingId mailingId)
        {
            Mailing mailing = (Mailing)helper.get(mailingId);

            if (mailingId is StandardMailingId)
            {
                Console.WriteLine("\nStandardMailing");
            }
            else if (mailingId is TransactionalMailingId)
            {
                Console.WriteLine("\nTransactionalMailing");
            }

            Console.WriteLine("\tID: " + mailing.objectId.id);
            Console.WriteLine("\tName: " + mailing.name);
            Console.WriteLine("\tCampaign: " + mailing.campaignId);
            Console.WriteLine("\tCreatedTime: " + mailing.createdTime);
            Console.WriteLine("\tDescription: " + mailing.description);
            Console.WriteLine("\tModifiedTime: " + mailing.modifiedTime);
            Console.WriteLine("\tOrganization: " + mailing.organizationId.id);
            Console.WriteLine("\tOwenr: " + mailing.ownerId.id);
            Console.WriteLine("\tFieldDelimiter: " + mailing.fieldDelimiter);
            Console.WriteLine("\tRowDelimiter: " + mailing.rowDelimiter);
            Console.WriteLine("\tMessageFormat: " + mailing.format[0]);
            Console.WriteLine("\tFromName: " + mailing.fromName);
            Console.WriteLine("\tHeader Size: " + mailing.header.Length);
            Console.WriteLine("\tTemppate: " + mailing.templateId.id);
            Console.WriteLine("\tBounce: " + mailing.bounceAddressId.id);
            Console.WriteLine("\tFrom: " + mailing.fromAddressId.id);
            Console.WriteLine("\tReply: " + mailing.replyAddressId.id);
            Console.WriteLine("\tBodyEncoding: " + mailing.bodyEncoding);
            Console.WriteLine("\tHeaderEncoding: " + mailing.headerEncoding);
            Console.WriteLine("\tMailngClass: " + mailing.mailingClassId.id);
            Console.WriteLine("\tPriority: " + mailing.priority);
            Console.WriteLine("\tServerErrorCode: " + mailing.serverErrorCode);
            Console.WriteLine("\tSererErrorMessage: " + mailing.serverErrorMessage);
            Console.WriteLine("\tStatus: " + mailing.status);
            Console.WriteLine("\tSubject: " + mailing.subject);
            Console.WriteLine("\tType: " + mailing.type);
            
            if (mailingId is StandardMailingId)
            {
                StandardMailing m = (StandardMailing) mailing;
                Console.WriteLine("\tIncludedTargets Size: " + m.includedTargetId.Length);
                Console.WriteLine("\tSeedLists Size: " + m.seedListId.Length);
                Console.WriteLine("\tSuppresionLists Size: " + m.suppressionListId.Length);
                Console.WriteLine("\tAttachment Size: " + mailing.attachmentId.Length);
                Console.WriteLine("\tContentBlock Size: " + mailing.contentBlockId.Length);
            }
            else if (mailingId is TransactionalMailingId)
            {
                TransactionalMailing m = (TransactionalMailing)mailing;
                Console.WriteLine("\tFormatField: " + m.formatFieldName);
                Console.WriteLine("\tMailingConfig: " + m.mailingConfig);
                Console.WriteLine("\tMessageType: " + m.messageType);
                Console.WriteLine("\tRecordStructure: " + m.recordStructure);
                Console.WriteLine("\tSenderNumber: " + m.senderNumber);
                Console.WriteLine("\tTarget: " + m.targetId.id);
            }
        }


        /* Launch a standard mailing
         */
        public bool launchMailing(StandardMailingId mailingId)
        {
            LaunchRequest request = new LaunchRequest();
            request.mailingId = mailingId ;

            return service.launch(request).success;
        }

        /* Schedule a standard mailing
         */
        public bool scheduleMailing(StandardMailingId mailingId)
        {
            ScheduleRequest request = new ScheduleRequest();
            request.mailingId = mailingId;

            return service.schedule(request).success;
        }

        /* Load a transactional mailing
         */

        public bool loadMailing(TransactionalMailingId txMailingId)
        {
            LoadRequest loadRequest = new LoadRequest();
            loadRequest.mailingId = txMailingId;

            return service.load(loadRequest).success;
        }

        /**
        * Fetch the handle for a tx mailing, that is to be used in the txnSend API.
        */
          public String getTxnMailingHandle(TransactionalMailingId mailingId)
          {
            // Create request and set mailingId
            GetTxnMailingHandleRequest request = new GetTxnMailingHandleRequest();
            request.Item = mailingId;
            return service.getTxnMailingHandle(request).handle;
          }

          /*
           * Uses TxnMailingHandle to send mail (Please refer the section 2. c in README.txt )
           */
          public bool txnSend(String mailingHandle)
          {
            // Create request and set mailingId
            TxnSendRequest request = new TxnSendRequest();
            request.handle = mailingHandle;


            // Create record based on the target created in TargetSample

            NameValuePair pair1 = new NameValuePair();
            pair1.name = "ID";
            pair1.value = "10";

            NameValuePair pair2 = new NameValuePair();
            pair2.name = "EMAIL_ADDRESS";
            pair2.value = "sample1@cxf.sample.com";

            NameValuePair pair3 = new NameValuePair();
            pair3.name = "NAME";
            pair3.value = "First Last";

            NameValuePair[] record1 = new NameValuePair[]{pair1,pair2,pair3};

            SendRecord sendRecord = new SendRecord();
            sendRecord.field = record1;

            // Add record and make call
            request.sendRecord = new SendRecord[] { sendRecord };

            bool success = false;
            try
            {
              success = service.txnSend(request).success;
              Console.WriteLine("Txsend Request was "+ (success ? "successful" : "unsuccessful" ));
            }
            catch (Exception e)
            {
              Console.WriteLine("Encountered a BadHandleFault. " + e.Message);
              Console.WriteLine("This indicates that the handle being used to attempt the txnSend call is incorrect.");
              Console.WriteLine("Please ensure that you send the correct handle using the getTxnMailingHandle() API.");
            }
            return success;
          }


        // Commenting out send functions since they are deprecated
        /* Send one record for transactional mailing.
         * 
         * txMailingId is the ID of transactional mailing whose data will be sent
         * return if send operation is successfule.
         */
        /*public bool sendOneRecord(TransactionalMailingId txMailingId)
        {
            // Create request
            SendRequest sendRequest = new SendRequest();
            sendRequest.mailingId = txMailingId;

            // Create one record based on the target created in TargetSample
            SendRecord record = new SendRecord();
            NameValuePair[] sendFields = new NameValuePair[3];

            NameValuePair filed_id = new NameValuePair();
            filed_id.name = "ID";
            filed_id.value = "10";
            sendFields[0] = filed_id;

            NameValuePair field_email = new NameValuePair();
            field_email.name = "email_address";
            field_email.value = "sample1@dotnet.sample.com";
            sendFields[1] = field_email;

            NameValuePair field_name = new NameValuePair();
            field_name.name = "attachmentName";
            field_name.value = "First Last";
            sendFields[2] = field_name;

            record.field = sendFields;

            // Set records
            SendRecord[] sendRecords = new SendRecord[1];
            sendRecords[0] = record;
            sendRequest.Items = sendRecords;

            // Make call
            SendResponse response = service.send(sendRequest);
            return response.success;
        }*/


        /* Send data using strings.
         * 
         * txMailingId is the ID of transactional mailing whose data will be sent.
         * return if send operation is successful
         */
        /*public bool sendData(TransactionalMailingId txMailingId)
        {
            // Get delimiter from mailing and use it for data string
            String fieldDelimiter = ((TransactionalMailing)helper.get(txMailingId)).fieldDelimiter;
            String rowDelimiter = ((TransactionalMailing)helper.get(txMailingId)).rowDelimiter;

            // Create request 
            SendRequest request = new SendRequest();
            request.mailingId = txMailingId;

            // Set data: data is rows of strings separated by delimiters
            String data = "20" + fieldDelimiter + "sample2@dotnet.sample.com" + fieldDelimiter + "FirstName LastName" + rowDelimiter;
            request.Items = new String[] {data};

            return service.send(request).success;
        }
*/
 
        /* Close a transactional mailing
         */
        public bool closeMailing(TransactionalMailingId txMailingId)
        {
            CloseRequest closeRequest = new CloseRequest();
            closeRequest.mailingId = txMailingId;

            return service.close(closeRequest).success;
        }


        /* 
         * Cancel a mailing
         */
        public bool cancelMailing(MailingId mailingId)
        {
            CancelRequest request = new CancelRequest();
            request.mailingId = mailingId;

            return service.cancel(request).success;
        }


        /* 
         * Pause a mailing
         */
        public bool pauseMailing(MailingId mailingId)
        {
            PauseRequest request = new PauseRequest();
            request.mailingId = mailingId;

            return service.pause(request).success;
        }


        /*
         * Resume a mailing
         */

        public bool resumeMailing(MailingId mailingId)
        {
            ResumeRequest request = new ResumeRequest();
            request.mailingId = mailingId;

            return service.resume(request).success;
        }

        /* List mailings by attachmentName: this should return one mailnig if attachmentName is matched.
         * 
         * attachmentName is the mailing attachmentName to be listed.
         */
        public void listByName(String name)
        {
            // Create filter and set default values
            MailingFilter filter = new MailingFilter();
            helper.setFilter(filter);

            // Create attachmentName condition
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = name;

            // Create filter request and set filter to request
            ListRequest request = new ListRequest();
            request.filter = filter;

            // Make call and print results
            printListResponse(service.list(request));
        }


        /* List mailings in an organization: this can be done by using empty filter.
         * The organization is the one from header.
         */
        public void listByOrganization()
        {
            // Create empty filter: no conditions, and set default attributes
            MailingFilter filter = new MailingFilter();
            helper.setFilter(filter);

            // Create list request and set the filter to request
            ListRequest request = new ListRequest();
            request.filter = filter;

            // Make call and print results
            printListResponse(service.list(request));
        }


        /* List mailings by mailing type and status
         */
        public void listByTypeAndStatus()
        {
            // Create a mailing filer and set default attributes
            MailingFilter filter = new MailingFilter();
            helper.setFilter(filter);
            
            // Create status condition: status in (ACTIVE, EDITING, COMPLETED)
            ArrayStringFilterCondition statusCondition = new ArrayStringFilterCondition();
            statusCondition.@operator = FilterArrayOperator.IN;
            statusCondition.value = new string[] {
                MailingStatus.ACTIVE.ToString(), 
                MailingStatus.EDITING.ToString(), 
                MailingStatus.COMPLETED.ToString()};
            filter.statusCondition = statusCondition;

            // Create type condition: type in (RECURRING, TRANSACTIONAL)
            ArrayStringFilterCondition typeCondition = new ArrayStringFilterCondition();
            typeCondition.@operator = FilterArrayOperator.IN;
            typeCondition.value = new string[] { MailingType.RECURRING.ToString(), MailingType.TRANSACTIONAL.ToString() };
            filter.typeCondition = typeCondition;

            // Create request and set filter to request
            ListRequest request = new ListRequest();
            request.filter = filter;

            printListResponse(service.list(request));
        }


        /* Print out list response: print ID and attachmentName
         */
        private void printListResponse(ListResponse response)
        {
            Console.WriteLine("\nList Response");
            foreach (ObjectId each in response.objectId)
            {
                Mailing mailing = (Mailing) helper.get(each);
                Console.WriteLine("\t" + each.id + " " + mailing.name);
            }
        }
    }
}
