﻿/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;

namespace ConsoleApplication1
{
    class SampleHelper
    {
        private MailingService service;

        public SampleHelper(MailingService service)
        {
            this.service = service;
        }


        /* Get one object based on its ID.
         * 
         * objectId is the object ID to retrieve.
         * returan an object whose ID is objectId.
         */
        public BaseObject get(ObjectId objectId)
        {
            ObjectId[] objectIds = new ObjectId[1];
            objectIds[0] = objectId;

            return get(objectIds)[0];
        }

        /* Get an array of objects based on given IDs.
         * 
         * objectIds is the list of IDs that user want to get.
         * return an array of BaseObjects.
         */
        public BaseObject[] get(ObjectId[] objectIds)
        {
            // Call server
            BatchGetResponse batchGetResponse = service.get(objectIds);

            // Create an array of BaseObject with the same size as the list of IDs.
            BaseObject[] objects = new BaseObject[objectIds.Length];

            // Go through each batchCreateResponse inside batch batchCreateResponse and get the object if it is successful.
            // Print out error message if any object is not retrieved successfully and set the object to null.
            for (int i = 0; i < batchGetResponse.success.Length; i++)
            {
                if (batchGetResponse.success[i])
                {
                    objects[i] = batchGetResponse.getResponse[i].baseObject;
                }
                else
                {
                    FaultDetail faultDetail = batchGetResponse.fault[i];
                    Console.WriteLine(faultDetail.faultCode + ": " + faultDetail.faultMessage);
                    objects[i] = null;
                }
            }

            return objects;
        }


        /* Create an object.
         * 
         * anObject is the object that will be created.
         * return the objectId.
         */
        public ObjectId create(BaseObject anObject)
        {
            BaseObject[] objects = new BaseObject[1];
            objects[0] = anObject;

            return create(objects)[0];
        }

        /* Create an array of objects.
         * 
         * objects is the object array that will be created.
         * return the objectId array for the created objects.
         */
        public ObjectId[] create(BaseObject[] objects)
        {
            // Call server
            BatchCreateResponse batchCreateResponse = service.create(objects);

            // Create an array to hold created ObjectIds
            ObjectId[] result = new ObjectId[objects.Length];

            // Go through each response inside batch response and get the objectId if it is successful.
            // Print out error message if any object is not created successfully and set the objectId to null.
            for (int i = 0; i < batchCreateResponse.success.Length; i++)
            {
                if (batchCreateResponse.success[i])
                {
                    result[i] = batchCreateResponse.createResponse[i].objectId;
                }
                else
                {
                    FaultDetail faultDetail = batchCreateResponse.fault[i];
                    Console.WriteLine(objects[i].GetType() + ": " + faultDetail.faultCode + ": " + faultDetail.faultMessage);
                    result[i] = null;
                }
            }

            return result;
        }


        /* Update an object.
         * 
         * anObject is the object that will be updated.
         */
        public void update(BaseObject anObject)
        {
            BaseObject[] objects = new BaseObject[1];
            objects[0] = anObject;

            service.update(objects);
        }


        /* Update an array of objects.
         * 
         * objects is the object array that will be updated.
         */
        public void update(BaseObject[] objects)
        {
            // Call server
            BatchUpdateResponse batchUpdateResponse = service.update(objects);

            // Go through each request and print out error message if update failed.
            for (int i = 0; i < batchUpdateResponse.success.Length; i++)
            {
                if (!batchUpdateResponse.success[i])
                {
                    FaultDetail faultDetail = batchUpdateResponse.fault[i];
                    Console.WriteLine(objects[i].GetType() + ": " + faultDetail.faultCode + ": " + faultDetail.faultMessage);
                }
            }
        }


        /* Delete an object.
         * 
         * objectId is the ID of the object that will be deleted.
         */
        public void delete(ObjectId objectId)
        {
            ObjectId[] objectIds = new ObjectId[1];
            objectIds[0] = objectId;

            delete(objectIds);
        }


        /* Delete an array of objects.
         * 
         * objectIds is the objectId array that will be deleted.
         */
        public void delete(ObjectId[] objectIds)
        {
            // Call server
            BatchDeleteResponse batchDeleteResponse = service.delete(objectIds);

            // Go through each request and print out error message if delete failed.
            for (int i = 0; i < batchDeleteResponse.success.Length; i++)
            {
                if (!batchDeleteResponse.success[i])
                {
                    FaultDetail faultDetail = batchDeleteResponse.fault[i];
                    Console.WriteLine(objectIds[i] + ": " + faultDetail.faultCode + ": " + faultDetail.faultMessage);
                }
            }
        }


        /* Set values to a filter: get the first page, 10 records per page, and using ascending order.
         */
        public void setFilter(BaseFilter filter)
        {
            // Set the order of returned records: ascending or descending
            filter.isAscending = true;
            filter.isAscendingSpecified = true;

            // Set the page number you want to get: starting from 0
            filter.pageNumber = 0;
            filter.pageNumberSpecified = true;

            // Set number of records per page
            filter.recordsPerPage = 10;
            filter.recordsPerPageSpecified = true;
        }


        /* Wait for a data source operation to be idle.
         * When adding records, the data source operation status will be IMPORTING
         * when user should not have other operation on the data source.
         * 
         * Wait for up to one minute since the data source records is small for sample code.
         */
        public void waitForIdle(DataSourceId dataSourceId)
        {
            long endTime = DateTime.Now.Second + 60; // up to one minute
            DataSource dataSource;

            while (DateTime.Now.Second < endTime)
            {
                dataSource = (DataSource) get(dataSourceId);
                if (!dataSource.operationStatus.Equals(DataSourceOperationStatus.IDLE))
                {
                    Thread.Sleep(1000); // sleep for one second
                }
                else
                {
                    break;
                }
            }
        }
    }
}
