<?php
/* Copyright 2009, StrongMail Systems, Inc.  All rights reserved. */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class DataSourceSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create an Internal Data Source with 3 columns: id, email_address, and name.
   *
   * @param dsname the name of Internal Data Source
   * @return the ID of Internal Data Source
   * @throws Exception any exception
   */
  public function createIDS($dsname)
  {
    // Create InternalDataSource and fill in properties.
    $dataSource = new InternalDataSource();
    $dataSource->description = "Internal Data Source from PHP";
    $dataSource->name = $dsname;

    // Specify the data source fields: id (primary key), email and name.
    // System will generate "unsub" and "email_status" fields
    $dataSource->field = array();
    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"INTEGER",
                                 "isPrimaryKey"=>true,
                                 "name"=>"id",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"EMAIL",
                                 "isPrimaryKey"=>false,
                                 "name"=>"email_address",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"TEXT",
                                 "isPrimaryKey"=>false,
                                 "name"=>"name",
                                 "writebackEnabled"=>false);

    // Create data source
    $request = SampleHelper::getSoapCreateRequestObject($dataSource, "InternalDataSource");
    $response = $this->service->create($request);
    $dataSourceId = $response->createResponse[0]->objectId;

    // Add some data
    $this->addRecordsInline($dataSourceId);

    return $dataSourceId;
  }

  /**
   * Create an External Data Source.
   *
   * @param edsName the name of the External Data Source
   * @return the EDS ID
   * @throws Exception any exception
   */
  public function createEDS($edsName)
  {
    $dataSource = new ExternalDataSource();
    $dataSource->allowRefreshAtLaunchTime = true;
    $dataSource->description = "External Data Source from PHP";
    $dataSource->enableLocalCopy = true;
    $dataSource->name = $edsName;
    $dataSource->sourceTableName = "ws_100k";
    $dataSource->writebackTable = "ws_100k";

    // Set connection info
    $info = new ConnectionInfo();

    $info->databaseName = "sm";
    $info->databaseType = "ORACLE";
    $info->hostname = "10.0.0.251";
    $info->password = "strongmail";
    $info->port = "1521";
    $info->username = "strongmail";
    $dataSource->connectionInfo = $info;

    // Set refresh frequency
    $daily = new DailyRefresh();
    $daily->startTime = time();
    $dataSource->dailyRefresh = $daily;

    // Set fields: need to make sure the fields match EDS columns
    $dataSource->field = array();
    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"INTEGER",
                                 "isPrimaryKey"=>true,
                                 "name"=>"ID",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"EMAIL",
                                 "isPrimaryKey"=>false,
                                 "name"=>"email",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"SMS",
                                 "isPrimaryKey"=>false,
                                 "name"=>"phone",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"STATUS",
                                 "fieldType"=>"TEXT",
                                 "isPrimaryKey"=>false,
                                 "name"=>"email_status",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"UNSUBSCRIBE",
                                 "fieldType"=>"TEXT",
                                 "isPrimaryKey"=>false,
                                 "name"=>"unsub",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"TEXT",
                                 "isPrimaryKey"=>false,
                                 "name"=>"first_name",
                                 "writebackEnabled"=>false);

    $dataSource->field[] = array("dataType"=>"DATA",
                                 "fieldType"=>"TEXT",
                                 "isPrimaryKey"=>false,
                                 "name"=>"last_name",
                                 "writebackEnabled"=>false);
    // Create data source
    $request = SampleHelper::getSoapCreateRequestObject($dataSource, "ExternalDataSource");
    $response = $this->service->create($request);
    $dataSourceId = $response->createResponse[0]->objectId;

    return $dataSourceId;
  }

  public function addRecordsInline($dataSourceId)
  {
    $addRecordsRequest = new AddDataSourceRecordsRequest();
    $addRecordsRequest->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                           $dataSourceId->id,
                                                                           "InternalDataSourceId");

    // Create records based on the datasource schema created in createIDS()
    $record1 = new DataSourceRecord();
    $record1->field = array();

    $idCol = new NameValuePair();
    $idCol->name = "id";
    $idCol->value = "1";
    $record1->field[] = $idCol;

    $emailCol = new NameValuePair();
    $emailCol->name = "email_address";
    $emailCol->value = "email_for_php@php.com";
    $record1->field[] = $emailCol;

    $nameCol = new NameValuePair();
    $nameCol->name = "name";
    $nameCol->value = "php_name";
    $record1->field[] = $nameCol;

    // Wait for data source status to be idle before adding
    $this->helper->waitForIdle($dataSourceId);

    // Add record to request and make call
    $addRecordsRequest->dataSourceRecord = array($record1);

    $this->service->addRecords($addRecordsRequest);
  }

  /**
   * Add records using a file to the datasource created in createIDS().
   *
   * @param dataSourceId the data source ID where records will be added to.
   * @param dataFile     the file that will be uploaded
   * @throws Exception any exception
   */
  public function addRecordsUsingFile($dataSourceId, $dataFile)
  {
    // Create request
    $addRecordsRequest = new AddDataSourceRecordsRequest();
    $addRecordsRequest->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                           $dataSourceId->id,
                                                                           "InternalDataSourceId");
    $addRecordsRequest->containsFieldNames = true;
    $addRecordsRequest->fieldDelimiter = ",";

    $addRecordsRequest->dataSourceRecordsAttachment = file_get_contents($dataFile);

    // Wait for data source status to be idle before adding
    $this->helper->waitForIdle($dataSourceId);

    $this->service->addRecords($addRecordsRequest);
  }

  /**
   * Remove the record added in addRecordsInline().
   *
   * @param dataSourceId the DataSourceId whose records will be removed.
   * @return the number of records that was removed.
   * @throws Exception any exception
   */
  public function removeRecordsInline($dataSourceId)
  {
    $request = new RemoveDataSourceRecordsRequest();
    $request->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                 $dataSourceId->id,
                                                                 "InternalDataSourceId");
    $request->matchFieldName = "name";
    $request->record = array();
    $request->record[] = "php_name";

    $response = $this->service->removeRecords($request);
    return $response->recordsRemoved;
  }

  /**
   * Retrieve and print out DataSource statistics.
   *
   * @param dataSourceId the data source ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrintStatistics($dataSourceId)
  {
    // Create request
    $request = new GetDataSourceStatisticsRequest();
    $request->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                 $dataSourceId->id,
                                                                 "InternalDataSourceId");

    // Make call
    $response = $this->service->getStatistics($request);

    print("\nStatistics for data source " . $dataSourceId->id . "\n");
    print("Total records: " . $response->totalRecords . "\n");
    print("Total invalid records: " . $response->totalInvalidRecords . "\n");
    print("Total malformed records: " . $response->totalMalformedRecords . "\n");
    print("Total unsub records: " . $response->totalUnsubscribedRecords . "\n");
  }

  /**
   * Get and print datasource object to standard out.
   *
   * @param dataSourceId the data source ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($dataSourceId)
  {
    $dataSources = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                                $dataSourceId->id,
                                                                                "InternalDataSourceId")));
    $dataSource = $dataSources[0];

    // Print data source to standard out
    print("\nDataSource:" . "\n");
    print("DataSourceId: " . $dataSource->objectId->id . "\n");
    print("Name: " . $dataSource->name . "\n");
    print("Description: " . $dataSource->description . "\n");
    print("CampaignId: " . $dataSource->campaignId . "\n");
    print("CreatedTime: " . $dataSource->createdTime . "\n");
    print("ModifiedTime: " . $dataSource->modifiedTime . "\n");
    print("OperationStatus: " . $dataSource->operationStatus . "\n");
    print("OrganizationId: " . $dataSource->organizationId->id . "\n");
    print("UserId: " . $dataSource->ownerId->id . "\n");

    // Print out field definition (data source schema)
    foreach ($dataSource->field as $field)
    {
      print($field->name . " " . $field->fieldType . " " . $field->dataType . "\n");
    }

    if ($dataSourceId instanceof ExternalDataSourceId)
    {
      $eds = $dataSource;
      print("AdvQuery: " . $eds->advancedQuery . "\n");
      print("DailyRefreshStartTime: " . $eds->dailyRefresh->startTime . "\n");
      print("Tablename: " . $eds->tableName . "\n");
      print("WriteBackTable: " . $eds->writebackTable . "\n");
      $info = $eds->connectionInfo;
      print("Connection: " .
            $info->databaseName . " " .
            $info->databaseType . " " .
            $info->hostname . " " .
            $info->port . " " .
            $info->username . " " .
            $info->password // Server does not return password
            . "\n");
    }
  }

  /**
   * Copy a data source using a new name.
   *
   * @param dataSourceId the data source ID that will be copied
   * @param newName      the name of new data source
   * @return the new InternalDataSource ID
   * @throws Exception any exception
   */
  public function copyDataSource($dataSourceId, $newName)
  {
    $request = new CopyDataSourceRequest();
    $request->fromId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                           $dataSourceId->id,
                                                           "InternalDataSourceId");
    $request->newName = $newName;
    $response = $this->service->copy($request);

    return $response->objectId;
  }

  /**
   * Update a data source with new name.
   *
   * @param dataSourceId the data source ID whose name will be updated
   * @param newName      the new name for the data source
   * @throws Exception any exception
   */
  public function updateDataSourceName($dataSourceId, $newName)
  {
    $dataSources = $this->helper->get(array($dataSourceId));
    $dataSource = $dataSources[0];
    $current_object_version = $current_ds->getResponse->baseObject->version;

    // Change name and update
    $dataSource->name = $newName;

    $request = SampleHelper::getSoapUpdateRequestObject($dataSource, "InternalDataSource");
    $response = $this->service->update($request);

    return $response->updateResponse->baseObject;
  }

  public function dataSourceGetStatistics($service, $dataSourceId)
  {
    $request = new GetDataSourceStatisticsRequest();
    $request->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                 $dataSourceId->id,
                                                                 "InternalDataSourceId");

    $response = $this->service->getStatistics($request);
    return $response;
  }

  /**
   * Remove duplicate records for a data source based on field "name". If duplicate records are
   * found, keep the latest ones.
   *
   * @param dataSourceId the InternalDataSourceId that will be deduped
   * @return if dedupe is successful
   * @throws Exception any exception
   */
  public function dedupeRecords($dataSourceId)
  {
    $request = new DedupeDataSourceRecordsRequest();
    $request->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                 $dataSourceId->id,
                                                                 "InternalDataSourceId");
    $request->option = "KEEP_NEWEST";
    $request->matchField = array("name");

    $response = $this->service->dedupeRecords($request);

    return $response->dedupeResponse->success;
  }

  /**
   * Export data source records to a file.
   *
   * @param dataSourceId the data source to be exported
   * @param toFile       the file to be written to
   * @throws Exception any exception
   */
  public function exportRecords($dataSourceId, $toFile)
  {
    // Create and fill in request
    $request = new ExportDataSourceRecordsRequest();
    $request->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                                 $dataSourceId->id,
                                                                 "InternalDataSourceId");
    $request->fieldDelimiter = "|";
    $request->rowDelimiter = "\n";

    // Export regular records. Use true to export malformed records.
    $request->useMalformedRecords = false;

    // Wait for data source status to be idle before exporting
    $this->helper->waitForIdle($dataSourceId);

    $response = $this->service->exportRecords($request);

    $fh = fopen($toFile, "wb");
    fwrite($fh, $response->data);
    fclose($fh);
  }

  public function dataSourceDelete($service, $dataSourceId)
  {
    $request = new DeleteRequest();
    $request->objectId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
                                                             $dataSourceId->id,
                                                             "InternalDataSourceId");

    $response = $this->service->delete($request);
    return $response;
  }
}
?>
