# Copyright 2014 StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'
require 'base64'

class DataSourceSample
  def initialize(svc)
    @svc = svc
  end


  #
  # Method to create an IDS. The IDS name
  # is in the opts hash (:ids_name). The IDS
  # created will contain a single field 'email'
  #
  def createIDS(ds_name)

    # Create InternalDataSource and fill in properties.
    dataSource = InternalDataSource.new
    dataSource.name = ds_name
    dataSource.description = "Internal Data Source from Ruby"

    # Specify the data source fields: id (primary key), email and name.
    # System will generate "unsub" and "email_status" fields
    fields = []

    primaryKey = DataSourceField.new
    primaryKey.dataType = "DATA"
    primaryKey.fieldType = "INTEGER"
    primaryKey.isPrimaryKey = true
    primaryKey.name = "id"
    primaryKey.writebackEnabled = false
    fields.push(primaryKey)

    email = DataSourceField.new
    email.dataType = "DATA"
    email.fieldType = "EMAIL"
    email.isPrimaryKey = false
    email.name = "email_address"
    email.writebackEnabled = false
    fields.push(email)

    name = DataSourceField.new
    name.dataType = "DATA"
    name.fieldType = "TEXT"
    name.isPrimaryKey = false
    name.name = "name"
    name.writebackEnabled = false
    fields.push(name)

    # Assign fields to data source
    dataSource.field = fields

    # Create data source and return result
    dataSourceId = SampleHelper::create(@svc, dataSource)

    # Add some data
    addRecordsInline(dataSourceId)

    return dataSourceId
  end


  # Create an External Data Source: Local copy
  #  edsName is the name of External Data Source
  #  return the EDS ID that is created
  def createEDS(edsName)
    # Set name, description, and local copy to true.
    eds = ExternalDataSource.new
    eds.name = edsName
    eds.description = "External Data Source from Ruby"
    eds.enableLocalCopy = true

    # Set connection info
    info = ExternalDataSource::ConnectionInfo.new
    info.databaseName = "DATABASE_NAME"
    info.databaseType = "ORACLE"
    info.hostname = "10.0.0.0"
    info.port = "1521"
    info.username = "USERNAME"
    info.password = "PASSWORD"
    eds.connectionInfo = info

    # Use daily refresh
    dailyRefresh = ExternalDataSource::DailyRefresh.new
    dailyRefresh.startTime = Time.new.strftime("%X")
    eds.dailyRefresh = dailyRefresh
    eds.allowRefreshAtLaunchTime = true
    eds.writebackTable = "WRITEBACK_TABLE"
    eds.sourceTableName = "SOURCE_TABLE"

    # Set fields: need to make sure the fields match EDS columns
    fields = []
    primaryKey = DataSourceField.new
    primaryKey.dataType = "DATA"
    primaryKey.fieldType = "INTEGER"
    primaryKey.isPrimaryKey = true
    primaryKey.name = "id"
    primaryKey.writebackEnabled = false
    fields.push(primaryKey)

    email = DataSourceField.new
    email.dataType = "DATA"
    email.fieldType = "EMAIL"
    email.isPrimaryKey = false
    email.name = "email"
    email.writebackEnabled = false
    fields.push(email)

    phone = DataSourceField.new
    phone.dataType = "DATA"
    phone.fieldType = "SMS"
    phone.isPrimaryKey = false
    phone.name = "phone"
    phone.writebackEnabled = false
    fields.push(phone)

    status = DataSourceField.new
    status.dataType = "STATUS"
    status.fieldType = "TEXT"
    status.isPrimaryKey = false
    status.name = "email_status"
    status.writebackEnabled = false
    fields.push(status)

    unsub = DataSourceField.new
    unsub.dataType = "UNSUBSCRIBE"
    unsub.fieldType = "TEXT"
    unsub.isPrimaryKey = false
    unsub.name = "unsub"
    unsub.writebackEnabled = false
    fields.push(unsub)

    fname = DataSourceField.new
    fname.dataType = "DATA"
    fname.fieldType = "TEXT"
    fname.isPrimaryKey = false
    fname.name = "first_name"
    fname.writebackEnabled = false
    fields.push(fname)

    lname = DataSourceField.new
    lname.dataType = "DATA"
    lname.fieldType = "TEXT"
    lname.isPrimaryKey = false
    lname.name = "last_name"
    lname.writebackEnabled = true
    fields.push(lname)

    eds.field = fields
    return SampleHelper::create(@svc, eds)
  end


  #
  # Add one record using inline to the data source created in createIDS().
  # dataSourceId is the data source ID where record will be added.
  #
  def addRecordsInline(dataSourceId)
    #setup the records to be inserted
    records = []

    # Create record based on the data source schema created in createIDS()
    values = []
    idCol = NameValuePair.new
    idCol.name = "id"
    idCol.value = "1"
    values.push(idCol)

    emailCol = NameValuePair.new
    emailCol.name = "email_address"
    emailCol.value= "email_for_ruby@samples.com"
    values.push(emailCol)

    nameCol = NameValuePair.new
    nameCol.name = "name"
    nameCol.value = "ruby_name"
    values.push(nameCol)

    records.push(values)

    # Add record to request and make call
    request = AddDataSourceRecordsRequest.new
    request.dataSourceId = dataSourceId
    request.containsFieldNames = false
    request.dataSourceRecord = records

    # Make call
    @svc.addRecords(request)
  end

   #
   # Upsert the record which has added at time of creation.
   # dataSourceId is the data source ID which record will be updated.
   #
   def upsertRecords(dataSourceId)
     records = []
     values = []
     idCol = NameValuePair.new
     idCol.name = "id"
     idCol.value = "1"
     values.push(idCol)

     emailCol = NameValuePair.new
     emailCol.name = "email_address"
     emailCol.value= "email_for_ruby_updated@samples.com"
     values.push(emailCol)

     nameCol = NameValuePair.new
     nameCol.name = "name"
     nameCol.value = "ruby_nameupdated"
     values.push(nameCol)
     records.push(values)

     # Upsert record request and make call
     request = UpsertDataSourceRecordsRequest.new
     request.dataSourceId = dataSourceId
     request.dataSourceRecord = records

     # Make call - upsertRecord api we do not need to check IDS status.
     @svc.upsertRecord(request)
   end

  #
  # Get records api
  # dataSourceId - data source ID of IDS .
  #
  def getRecords(dataSourceId)

    # Specify the get criteria
    matchFields = []
    idCond = NameValuePair.new
    idCond.name = "id"
    idCond.value = "1"
    matchFields.push(idCond)

    # Upsert record request and make call
    request = GetDataSourceRecordsRequest.new
    request.dataSourceId = dataSourceId
    request.matchFields = matchFields

    # Make call - getRecords api we do not need to check IDS status.
    response = @svc.getRecords(request);

    puts "Matching records found: " + response.dataSourceRecord.size.to_s()
  end

  # Add records using a file to the data source created in createIDS().
  #  dataSourceId is the data source ID where records will be added.
  #  dataFile is the file that will be uploaded.
  def addRecordsUsingFile(dataSourceId, dataFile)
    request = AddDataSourceRecordsRequest.new
    request.dataSourceId = dataSourceId
    request.containsFieldNames = true
    request.fieldDelimiter = ","

    # Read file and add to request
    request.dataSourceRecordsAttachment =
            Base64.encode64(File.open(dataFile, "r") {|io| io.read})

    # Make call
    @svc.addRecords(request)
  end


  # Remove the record added in addRecordsInline().
  #  dataSourceId is the data source ID whose records will be removed.
  #  return the number of records that was removed.
  def removeRecordsInline(dataSourceId)
    request = RemoveDataSourceRecordsRequest.new
    request.dataSourceId = dataSourceId
    request.matchFieldName = "name"

    # Name column will match these values
    request.record = ["ruby_name"]

    return @svc.removeRecords(request).recordsRemoved
  end


  # Export a data source records to a file.
  #  dataSourceId is the data source to be exported.
  #  toFile is the file to be written to.
  def exportRecords(dataSourceId, toFile)
    request = ExportDataSourceRecordsRequest.new
    request.dataSourceId = dataSourceId
    request.fieldDelimiter = "|"
    request.rowDelimiter = "\n"

    # Export regular records. Use true to export malformed records.
    request.useMalformedRecords = false

    # Wait for data source status to be idle before exporting
    waitForIdle(dataSourceId)

    # Make call and save data to file
    response = @svc.exportRecords(request)

    # Decode the content and write to a file
    str = Base64.decode64(response.data)
    File.open(toFile, "w") {|io| io.write(Base64.decode64(str))}
  end


  # Remove duplicate records for a data source based on field "name".
  # If duplicate records are found, keep the latest ones.
  #  idsId the InternalDataSourceId that will be deduped
  #  if dedupe is successful
  def dedupeRecords(dataSourceId)
    request = DedupeDataSourceRecordsRequest.new
    request.dataSourceId = dataSourceId
    request.option = "KEEP_NEWEST"
    request.matchField = ["name"]

    return @svc.dedupeRecords(request).success
  end


  # Retrieve and print out an data source statistics.
  #  dataSourceId is the data source ID that will be retrieved and printed.
  def getAndPrintStatistics(dataSourceId)
    # Create request
    request = GetDataSourceStatisticsRequest.new
    request.dataSourceId = dataSourceId

    # Make call
    response = @svc.getStatistics(request)

    # Print out statistics
    puts "\nStatistics for data source " + dataSourceId.id
    puts "Total records: " + response.totalRecords.to_s
    puts "Total invalid records: " + response.totalInvalidRecords.to_s
    puts "Total malformed records: " + response.totalMalformedRecords.to_s
    puts "Total unsub records: " + response.totalUnsubscribedRecords.to_s
  end


  def getAndPrint(dataSourceId)
    dataSource = SampleHelper::get(@svc, dataSourceId)
    puts dataSource.class
    puts "\tID: " + dataSource.objectId.id
    puts "\tName: " + dataSource.name
    puts "\tDescription: " + dataSource.description
    puts "\tCreated Time: " + dataSource.createdTime.to_s()
    puts "\tModified Time: " + dataSource.modifiedTime.to_s()
    puts "\tOperationStatus: " + dataSource.operationStatus
    puts "\tOrganization: " + dataSource.organizationId.id
    puts "\tUserId: " + dataSource.ownerId.id
    puts "\tFields: "

    for fld in dataSource.field
      puts "\t\t" + fld.name + " " + fld.fieldType
    end

    # EDS specific
    if dataSourceId.instance_of?(ExternalDataSourceId)
      puts "\tLocal Copy: " + dataSource.enableLocalCopy.to_s
      puts "\tDaily Refresh: " + dataSource.dailyRefresh.startTime.to_s
      puts "\tConnection: " +
              dataSource.connectionInfo.databaseName + " " +
              dataSource.connectionInfo.databaseType + " " +
              dataSource.connectionInfo.hostname + " " +
              dataSource.connectionInfo.port + " " +
              dataSource.connectionInfo.username
    end
  end


  # Updatea a data source with new name.
  #  dataSourceId is the data source ID whose name will be updated.
  #  newName is the new name for the data source.
  def updateDataSourceName(dataSourceId, newName)
    # Get the data source first
    dataSource = SampleHelper::get(@svc, dataSourceId)
    dataSource.name = newName

    # Update the data source with new name
    SampleHelper::update(@svc, dataSource)
  end


  # Copy a data source using a new name.
  #  dataSourceId is the data source ID that will be copied.
  #  newName is the name of new data source.
  #  return the new data source ID.
  def copyDataSource(dataSourceId, newName)
    # Create copy request
    request = CopyDataSourceRequest.new
    request.fromId = dataSourceId
    request.newName = newName

    return @svc.copy(request).objectId;
  end

  # Refresh External Data Source records.
  #  edsId is the EDS ID to be refreshed.
  #  return if refresh is successful.
  def refreshRecords(edsId)
    request = RefreshRecordsRequest.new
    request.dataSourceId = edsId
    return @svc.refreshRecords(request).success
  end


  # Cancel refreshing External Data Source records.
  #  edsId is the EDS ID to be refreshed.
  #  return if cancelling is successful.
  def cancelRefreshRecords(edsId)
    request = CancelRefreshRecordsRequest.new
    request.dataSourceId = edsId
    return @svc.cancelRefreshRecords(request).success
  end


  # List internal data source by a name.
  # It will return zero or one data source for an organization.
  #  name is the data source name that will be listed
  #  return the internal data source ID that matches the give name.
  def listInternalDataSourcesByName(name)
    # Create filter
    filter = DataSourceFilter.new

    # Set name condition
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = name
    filter.nameCondition = nameCondition
    setAttributeAndTypeToFilter(filter)

    return SampleHelper::list(@svc, filter)[0]
  end


  # List internal data source belonging to a user.
  def listInternalDataSourcesByUser(userId)
    # Create list request and filter
    filter = DataSourceFilter.new

    # Set user condition
    userCondition = ScalarIdFilterCondition.new
    userCondition.operator = "EQUAL"
    userCondition.value = userId
    filter.userIdCondition = userCondition
    setAttributeAndTypeToFilter(filter)

    # Make request and print out all IDs
    objectIds = @svc.list(filter).objectId

    puts "\nGot the following IDSs for User" + userId.id + ": "
    for id in objectIds
      print id.id + ","
    end
  end


  # Wait for a data source operation to be idle.
  # When adding records, the data source operation status will be IMPORTING
  # when user should not have other operation on the data source.
  # Wait for up to one minute since the data source records is small for sample code.
  def waitForIdle(dataSourceId)
    endTime = Time.now.to_i + 60 #  up to one minute
    while Time.now.to_i < endTime
      dataSource = SampleHelper::get(@svc, dataSourceId)
      if dataSource.operationStatus != "IDLE"
        sleep(5)
      else
        break
      end
    end
  end


  # Set filter type to internal.
  def setAttributeAndTypeToFilter(filter)
    # Set type condition: required for data source
    typeCondition = ScalarStringFilterCondition.new
    typeCondition.operator = "EQUAL"
    typeCondition.value = "INTERNAL"
    filter.typeCondition = typeCondition;

    SampleHelper::setFilter(filter);
  end

end
