# Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class MailingSample
  def initialize(svc, bounceId, fromId, replyId)
    @svc = svc
    @bounceId = bounceId
    @fromId = fromId
    @replyId = replyId
  end


  # Create a recurring mailing with no excluded target. Use only one of instead of list of
  #  target, seed list, suppresion list, attachment and content blocks.
  #  mailingName is the name of the mailing.
  #  targetId is the included target ID used for the mailing.
  #  seedListId is the seed list ID used with this mailing.
  #  suppressionListId is the suppresion list ID used with this mailing.
  #  attachmentId is the attachment ID used with this mailing.
  #  contentBlockId is the content block ID used with this mailing.
  #  templateId is the template ID used with this mailing.
  #  return the ID of created standard mailing.
  def createStandardMailing(
          mailingName, targetId, seedListId, suppressionListId,
          attachmentId, contentBlockId, templateId)
    mailing = StandardMailing.new
    setMailingCommonFields(mailing, mailingName, @bounceId, @fromId, @replyId, templateId)

    mailing.description = "StandardMailing from Ruby"
    mailing.eliminateDuplicates = true
    mailing.type = "RECURRING"

    # Set schedule: daily recurrence with every two days
    daily = SchedulableMailing::Schedule::Recurrence::DailyRecurrence.new
    daily.interval = 2

    recurrence = SchedulableMailing::Schedule::Recurrence.new
    recurrence.endDate = Date.new
    recurrence.dailyRecurrence = daily

    schedule = SchedulableMailing::Schedule.new
    schedule.startDateTime = DateTime::civil(2020, 1, 11, 1, 1, 1)
    schedule.recurrence = recurrence
    mailing.schedule = schedule

    # Set dependencies
    mailing.attachmentId = [attachmentId]
    mailing.contentBlockId = [contentBlockId]
    mailing.includedTargetId = [targetId]
    mailing.seedListId = [seedListId]
    mailing.suppressionListId = [suppressionListId]

    return SampleHelper::create(@svc, mailing)
  end

# Create a one time mailing with no excluded target. Use only one of instead of list of
  #  target, seed list, suppresion list, attachment and content blocks.
  #  mailingName is the name of the mailing.
  #  targetId is the included target ID used for the mailing.
  #  seedListId is the seed list ID used with this mailing.
  #  suppressionListId is the suppresion list ID used with this mailing.
  #  attachmentId is the attachment ID used with this mailing.
  #  contentBlockId is the content block ID used with this mailing.
  #  templateId is the template ID used with this mailing.
  #  return the ID of created standard mailing.
  def createOneTimeStandardMailing(
          mailingName, targetId, seedListId, suppressionListId,
          attachmentId, contentBlockId, templateId)
    mailing = StandardMailing.new
    setMailingCommonFields(mailing, mailingName, @bounceId, @fromId, @replyId, templateId)

    mailing.description = "StandardMailing from Ruby"
    mailing.eliminateDuplicates = true
    mailing.type = "ONE_TIME"

    schedule = SchedulableMailing::Schedule.new
    schedule.startDateTime = DateTime::civil(2020, 1, 11, 1, 1, 1)
    mailing.schedule = schedule

    # Set dependencies
    mailing.attachmentId = [attachmentId]
    mailing.contentBlockId = [contentBlockId]
    mailing.includedTargetId = [targetId]
    mailing.seedListId = [seedListId]
    mailing.suppressionListId = [suppressionListId]

    return SampleHelper::create(@svc, mailing)
  end


  # Create a transactional mailing.
  #  mailingName is the name of the mailing.
  #  targetId is the target used for the mailing.
  #  templateId is the template used for the mailing.
  #  contentBlockId is the content block used for the mailing.
  #  templateId is the template used in the mailing.
  #  return the ID of created transactional mailing.
  def createTransactionalMailing(mailingName, targetId, templateId)
    txMailing = TransactionalMailing.new
    setMailingCommonFields(txMailing, mailingName, @bounceId, @fromId, @replyId, templateId)

    txMailing.description = "TransactionalMailing from Ruby"
    txMailing.type = "TRANSACTIONAL"
    txMailing.messageType = "EMAIL"
    txMailing.targetId = targetId

    return SampleHelper::create(@svc, txMailing)
  end


  # Set mailing properties common to both standard and transactional mailings.
  def setMailingCommonFields(mailing, mailingName, bounceId, fromId, replyId, templateId)
    mailing.bodyEncoding = "BASE64"
    mailing.bounceAddressId = bounceId
    mailing.fieldDelimiter = "::"
    mailing.fromAddressId = fromId
    mailing.fromName = "From Name"
    mailing.headerEncoding = "EIGHT_BIT"
    mailing.isApproved = true
    mailing.isCompliant = true
    mailing.name = mailingName
    mailing.outputBodyCharSet = "ASCII"
    mailing.outputHeaderCharSet = "UTF-8"
    mailing.priority = "NORMAL"
    mailing.replyAddressId = replyId
    mailing.rowDelimiter = "\n"
    mailing.subject = "mailing subject"
    mailing.templateId = templateId
    mailing.format = ["HTML"]
    mailing.header = ["mailing header"]
    mailing.forward2FriendOffer = ""
    mailing.forward2FriendOfferTrackingOption = "NONE"
  end


  # Get and print out a mailing.
  #  mailingId is the ID of the mailing to be printed.
  def getAndPrint(mailingId)
    mailing = SampleHelper::get(@svc, mailingId)
    puts mailing.class
    puts "\tID: " + mailing.objectId.id
    puts "\tName: " + mailing.name
    puts "\tCreated Time: " + mailing.createdTime.to_s()
    puts "\tDescription: " + mailing.description
    puts "\tModified Time: " + mailing.modifiedTime.to_s()
    puts "\tOrganization: " + mailing.organizationId.id
    puts "\tUserId: " + mailing.ownerId.id
    puts "\tFieldDelimiter: " + mailing.fieldDelimiter
    puts "\tRowDelimiter: " + mailing.rowDelimiter
    puts "\tMessageFormat: " + mailing.format[0].to_s
    puts "\tFromName: " + mailing.fromName
    puts "\tHeader Size: " + mailing.header[0]
    puts "\tTemppate: " + mailing.templateId.id
    puts "\tBounce: " + mailing.bounceAddressId.id
    puts "\tFrom: " + mailing.fromAddressId.id
    puts "\tReply: " + mailing.replyAddressId.id
    puts "\tBodyEncoding: " + mailing.bodyEncoding.to_s
    puts "\tHeaderEncoding: " + mailing.headerEncoding.to_s
    puts "\tMailngClass: " + mailing.mailingClassId.id
    puts "\tPriority: " + mailing.priority.to_s
    puts "\tServerErrorCode: " + mailing.serverErrorCode.to_s
    puts "\tSererErrorMessage: " + mailing.serverErrorMessage.to_s
    puts "\tStatus: " + mailing.status.to_s
    puts "\tSubject: " + mailing.subject
    puts "\tType: " + mailing.type.to_s

    if (mailingId.instance_of?(StandardMailingId))
      puts "\tIncludedTargets: " + mailing.includedTargetId[0].id
      puts "\tSeedLists: " + mailing.seedListId[0].id
      puts "\tSuppresionLists: " + mailing.suppressionListId[0].id
      puts "\tAttachment: " + mailing.attachmentId[0].id
      puts "\tContentBlock: " + mailing.contentBlockId[0].id
    elsif (mailingId.instance_of?(TransactionalMailingId))
      puts "\tFormatField: " + mailing.formatFieldName.to_s
      puts "\tMailingConfig:" + mailing.mailingConfig.to_s
      puts "\tMessageType: " + mailing.messageType.to_s
      puts "\tRecordStructure: " + mailing.recordStructure.to_s
      puts "\tSenderNumber: " + mailing.senderNumber.to_s
      puts "\tTarget: " + mailing.targetId.id
    end
  end


  # Launch a standard mailing
  def launchMailing(mailingId)
    request = LaunchRequest.new
    request.mailingId = mailingId

    return @svc.launch(request).success
  end

 # Schedule a standard mailing
  def scheduleMailing(mailingId)
    request = ScheduleRequest.new
    request.mailingId = mailingId

    return @svc.schedule(request).success
  end

  # Load a transactional mailing
  def loadMailing( txMailingId)
    request = LoadRequest.new
    request.mailingId = txMailingId

    return @svc.load(request).success
  end

  def getTxnMailingHandle( txMailingId)
     # Create request and set mailingId
     request = GetTxnMailingHandleRequest.new
     request.mailingId = txMailingId

     return @svc.getTxnMailingHandle(request).handle
  end

  def txnSend(mailingHandle)
     # Create request and set mailingId
     request = TxnSendRequest.new
     request.handle = mailingHandle;

     # Create record based on the target created in TargetSample
     record = SendRecord.new;

     field_id = NameValuePair.new
     field_id.name = "ID"
     field_id.value = "10"

     field_email = NameValuePair.new
     field_email.name = "email_address"
     field_email.value = "sample1@ruby.sample.com"

     field_name = NameValuePair.new
     field_name.name = "attachmentName"
     field_name.value = "First Last"

     # Set records
     record = [field_id, field_email, field_name]
     request.sendRecord = [record]

     return @svc.txnSend(request).success;
  end



#Commenting out send functions since they are deprecated
  # Send one record for transactional mailing.
  #  txMailingId is the ID of transactional mailing whose data will be sent
  #  return if send operation is successfule.
#  def sendOneRecord(txMailingId)
#    # Create request
#    request = SendRequest.new
#    request.mailingId = txMailingId
#
#    # Create one record based on the target created in TargetSample
#    field_id = NameValuePair.new
#    field_id.name = "ID"
#    field_id.value = "10"
#
#    field_email = NameValuePair.new
#    field_email.name = "email_address"
#    field_email.value = "sample1@ruby.sample.com"
#
#    field_name = NameValuePair.new
#    field_name.name = "attachmentName"
#    field_name.value = "First Last"
#
#    # Set records
#    record = SendRecord.new
#    record = [field_id, field_email, field_name]
#    request.sendRecord = [record]
#
#    return @svc.send(request).success
#  end
#
#
#  # Send data using strings.
#  #  txMailingId is the ID of transactional mailing whose data will be sent.
#  #  return if send operation is successful
#  def sendData(txMailingId)
#    # Get delimiter from mailing and use it for data string
#    mailing = SampleHelper::get(@svc, txMailingId)
#    fieldDelimiter = mailing.fieldDelimiter
#    rowDelimiter = mailing.rowDelimiter
#
#    # Create request
#    request = SendRequest.new
#    request.mailingId = txMailingId
#
#    # Set data: data is rows of strings separated by delimiters
#    data = "20" + fieldDelimiter + "sample2@dotnet.sample.com" + fieldDelimiter + "FirstName LastName" + rowDelimiter
#    request.sendData = data
#
#    return @svc.send(request).success;
#  end



  # Close a transactional mailing
  def closeMailing(txMailingId)
    request = CloseRequest.new
    request.mailingId = txMailingId

    return @svc.close(request).success
  end


  # Cancel a mailing
  def cancelMailing(mailingId)
    request = CancelRequest.new
    request.mailingId = mailingId

    return @svc.resume(request).success
  end


  # Pause a mailing
  def pauseMailing(mailingId)
    request = PauseRequest.new
    request.mailingId = mailingId

    return @svc.pause(request).success
  end


  # Resume a mailing
  def resumeMailing(mailingId)
    request = ResumeRequest.new
    request.mailingId = mailingId

    return @svc.resume(request).success
  end

  
  # List mailings by attachmentName: this should return one mailnig if name is matched.
  #  name is the mailing name to be listed.
  def listByName(name)
    # Create list filter
    filter = MailingFilter.new
    SampleHelper::setFilter(filter)

    # Set name condition
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = name

    filter.nameCondition = nameCondition
    printListResponse(SampleHelper::list(@svc, filter))
  end

  
  # List mailings in an organization: this can be done by using empty filter.
  # The organization is the one from header.
  def listByOrganization()
    # Create list filter
    filter = MailingFilter.new
    SampleHelper::setFilter(filter)

    printListResponse(SampleHelper::list(@svc, filter))
  end


  # List mailings by mailing type and status
  def listByTypeAndStatus()
    # Create list filter
    filter = MailingFilter.new
    SampleHelper::setFilter(filter)

    # Create status condition: status in (ACTIVE, EDITING, COMPLETED)
    statusCondition = ArrayStringFilterCondition.new
    statusCondition.operator = "IN"
    statusCondition.value = ["ACTIVE", "EDITING", "COMPLETED"]
    filter.statusCondition = statusCondition

    # Create type condition: type in (RECURRING, TRANSACTIONAL)
    typeCondition = ArrayStringFilterCondition.new
    typeCondition.operator = "IN"
    typeCondition.value = ["RECURRING", "TRANSACTIONAL"]
    filter.typeCondition = typeCondition

    printListResponse(SampleHelper::list(@svc, filter))
  end

  
  # Print out list response: print ID and attachmentName
  def printListResponse(objectIds)
    puts "\nList Response: "
    for id in objectIds
      mailing = SampleHelper::get(@svc, id)
      puts "\t" + id.id + " " + mailing.name
    end
  end


  # Wait for mailing to become a desired status
  def waitForStatus(mailingId, desiredStatus)
    status = SampleHelper::get(@svc, mailingId).status

    # Return immediately if desired status is received, or if any permanent
    # status is encountered.
    while status != desiredStatus &&
          status != "ERROR" &&
          status != "FAILED" &&
          status != "COMPLETED" &&
          status != "COMPLETED_DEFERRED" &&
          status != "CANCELED" &&
          status != "SPEC_CANCELED"

      puts "waitForStatus on " + mailingId.id + ": " + status + "!=" + desiredStatus
      sleep(10);
      status = SampleHelper::get(@svc, mailingId).status
    end

    puts "waitForStatus on " + mailingId.id + " final: " + status
    return status;
  end



end
