/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.endpoint.ClientImpl;
import org.apache.cxf.endpoint.Endpoint;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.handler.WSHandlerConstants;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.MailingService_Service;
import com.sun.xml.ws.developer.SchemaValidationFeature;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Map;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;
import javax.xml.namespace.QName;
import javax.xml.ws.Binding;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPBinding;


public class ClientSetup
{
  /**
   * Service name constant: defined in WSDL
   */
  private static String SERVICE_NAME = "MailingService";

  /**
   * Schema namespace: defined in WSDL
   */
  private String schemaNS;

  /**
   * QName for the service
   */
  private final QName SERVICE_QNAME;

  /**
   * By default do not use MTOM since there is a bug in JAXB library: see:
   * https://jaxb.dev.java.net/issues/show_bug.cgi?id=588
   */
  private boolean useMtom = false;

  public ClientSetup(String serviceNS, String schemaNS)
  {
    this.schemaNS = schemaNS;
    SERVICE_QNAME = new QName(serviceNS, SERVICE_NAME);
  }

  /**
   * Create service using give WSDL file and endpoint. If endpointAddress is null, the one from WSDL
   * will be used.
   *
   * @param wsdlFile        WSDL to be used
   * @param endpointAddress endpoint address if user wants to use something different from WSDL
   * @return StrongMail MailingService
   * @throws Exception any exception
   */
  public MailingService createService(File wsdlFile, String endpointAddress) throws Exception
  {
    URL wsdlURL = wsdlFile.toURI().toURL();

    // Create service with schema validation on
    MailingService_Service mss = new MailingService_Service(wsdlURL, SERVICE_QNAME);
    SchemaValidationFeature feature = new SchemaValidationFeature();
    MailingService svc = mss.getMailingServicePort(feature);

    // Set address either from WSDL or from user input
    String address;
    if (endpointAddress != null)
    {
      address = endpointAddress;
    }
    else
    {
      Endpoint endpoint = ClientProxy.getClient(svc).getEndpoint();
      address = endpoint.getEndpointInfo().getAddress();
    }

    // Set endpoint
    ((BindingProvider)svc).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
                                                   address);

    // Set MTOM on/off
    Binding binding = ((BindingProvider)svc).getBinding();
    ((SOAPBinding)binding).setMTOMEnabled(useMtom);

    return svc;
  }

  /**
   * Use interceptor to set username, password, organizationName, and optionally subOrganizationId.
   *
   * @param svc               MailingService client
   * @param username          username to login
   * @param password          password for the user
   * @param organizationName  organization name to login
   * @param subOrganizationId sub-organization ID to login. Null for default root.
   * @param isSSO
   * @throws Exception any exception
   */
  public void applyInterceptor(MailingService svc,
                               String username,
                               String password,
                               String organizationName,
                               Long subOrganizationId,
                               boolean isSSO) throws Exception
  {

    // WSSE standard username token
    Map<String, Object> outprops = new HashMap<String, Object>();
    outprops.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);

    // Set username and password
    outprops.put(WSHandlerConstants.USER, username);
    outprops.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_TEXT);
    outprops.put(WSHandlerConstants.PW_CALLBACK_REF, new ClientPasswordCallbackHandler(password));

    // Create client interceptor
    AuthenticationInterceptor authenticationInterceptor =
      new AuthenticationInterceptor(schemaNS, outprops, organizationName, subOrganizationId, isSSO);
    authenticationInterceptor.setAllowMTOM(useMtom);

    // Set client to use HTTPS with/without checking certificate
    ClientImpl cli = (ClientImpl)ClientProxy.getClient(svc);

    // Use HTTPS and validate certificate
    useHttpsWithCertificate(cli);

    // Set interceptor
    Endpoint endpoint = cli.getEndpoint();
    endpoint.getOutInterceptors().add(authenticationInterceptor);
  }

  /**
   * Use HTTPS protocol and check certificate. Read key store, password from system properties.
   *
   * @param client web service client
   * @throws Exception any exception
   */
  private void useHttpsWithCertificate(Client client) throws Exception
  {
    String keystore = System.getProperty("javax.net.ssl.trustStore");
    String keystorePassword = System.getProperty(
      "javax.net.ssl.trustStorePassword");

    KeyStore ks = KeyStore.getInstance("JKS");
    ks.load(new FileInputStream(keystore), keystorePassword.toCharArray());

    TrustManagerFactory tmf = TrustManagerFactory.getInstance(
      TrustManagerFactory.getDefaultAlgorithm());
    tmf.init(ks);
    TrustManager[] tms = tmf.getTrustManagers();

    TLSClientParameters param = new TLSClientParameters();
    param.setSecureSocketProtocol("TLS");
    param.setTrustManagers(tms);

    HTTPConduit https = (HTTPConduit)client.getConduit();
    https.setTlsClientParameters(param);
  }

  /**
   * Use HTTPS protocol and accept all certificates.
   *
   * @param client web service client
   * @throws Exception any exception
   */
  private void useHttpsIgnoreCertificate(Client client) throws Exception
  {
    TrustManager[] trustAllCerts = new TrustManager[]{
      new X509TrustManager()
      {
        public X509Certificate[] getAcceptedIssuers()
        {
          return null;
        }

        public void checkClientTrusted(X509Certificate[] certs,
                                       String authType)
        {
          // No implementation
        }

        public void checkServerTrusted(X509Certificate[] certs,
                                       String authType)
        {
          // No implementation
        }
      }
    };

    TLSClientParameters param = new TLSClientParameters();
    param.setSecureSocketProtocol("TLS");
    param.setTrustManagers(trustAllCerts);
    param.setDisableCNCheck(true);

    HTTPConduit https = (HTTPConduit)client.getConduit();
    https.setTlsClientParameters(param);
  }
}
