/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import org.apache.commons.io.IOUtils;
import org.apache.cxf.attachment.AttachmentDataSource;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddDataSourceRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CancelRefreshRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CopyDataSourceRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSource;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceDataType;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceDedupeOption;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceField;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceFieldType;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceRecord;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DatabaseType;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DedupeDataSourceRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ExportDataSourceRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ExportRecordsResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ExternalDataSource;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ExternalDataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetDataSourceRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetDataSourceRecordsResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetDataSourceStatisticsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetDataSourceStatisticsResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSource;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceExtn;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceExtnId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NameValuePair;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RefreshRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RemoveDataSourceRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.UpsertDataSourceRecordsRequest;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.activation.DataHandler;


public class DataSourceSample
{
  private MailingService service;
  private SampleHelper helper;

  public DataSourceSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper(service);
  }

  /**
   * Create an Internal Data Source with 3 columns: id, email_address, and name.
   *
   * @param dsname the name of Internal Data Source
   * @return the ID of Internal Data Source
   * @throws Exception any exception
   */
  public InternalDataSourceId createIDS(String dsname)
    throws Exception
  {
    // Create InternalDataSource and fill in properties.
    InternalDataSource dataSource = new InternalDataSource();
    dataSource.setDescription("Internal Data Source from CXF");
    dataSource.setName(dsname);

    // Specify the data source fields: id (primary key), email and name.
    // System will generate "unsub" and "email_status" fields
    DataSourceField primaryKey = new DataSourceField();
    primaryKey.setDataType(DataSourceDataType.DATA);
    primaryKey.setFieldType(DataSourceFieldType.INTEGER);
    primaryKey.setIsPrimaryKey(true);
    primaryKey.setName("id");
    primaryKey.setWritebackEnabled(false);
    dataSource.getField().add(primaryKey);

    DataSourceField email = new DataSourceField();
    email.setDataType(DataSourceDataType.DATA);
    email.setFieldType(DataSourceFieldType.EMAIL);
    email.setIsPrimaryKey(false);
    email.setName("email_address");
    email.setWritebackEnabled(false);
    dataSource.getField().add(email);

    DataSourceField name = new DataSourceField();
    name.setDataType(DataSourceDataType.DATA);
    name.setFieldType(DataSourceFieldType.TEXT);
    name.setIsPrimaryKey(false);
    name.setName("name");
    name.setWritebackEnabled(false);
    dataSource.getField().add(name);

    // Create data source
    InternalDataSourceId dataSourceId = (InternalDataSourceId)helper.create(dataSource);

    // Add some data
    addRecordsInline(dataSourceId);

    return dataSourceId;
  }


  /**
   * Create an External Data Source.
   *
   * @param edsName the name of the External Data Source
   * @return the EDS ID
   * @throws Exception any exception
   */
  public ExternalDataSourceId createEDS(String edsName) throws Exception
  {
    ExternalDataSource dataSource = new ExternalDataSource();
    dataSource.setAllowRefreshAtLaunchTime(true);
    dataSource.setDescription("External Data Source from CXF");
    dataSource.setEnableLocalCopy(true);
    dataSource.setName(edsName);
    dataSource.setSourceTableName("YOUR_SOURCE_TABLE");
    dataSource.setWritebackTable("YOUR_TABLE_TO_WRITE_BACK");

    // Set connection info
    ExternalDataSource.ConnectionInfo info = new ExternalDataSource.ConnectionInfo();

    info.setDatabaseName("DATABASE_NAME");
    info.setDatabaseType(DatabaseType.ORACLE);
    info.setHostname("10.0.0.0");
    info.setPassword("PASSWORD");
    info.setPort("1521");
    info.setUsername("USERNAME");
    dataSource.setConnectionInfo(info);

    // Set refresh frequency
    ExternalDataSource.DailyRefresh daily = new ExternalDataSource.DailyRefresh();
    daily.setStartTime(new Date());
    dataSource.setDailyRefresh(daily);

    // Set fields: need to make sure the fields match EDS columns
    DataSourceField primaryKey = new DataSourceField();
    primaryKey.setDataType(DataSourceDataType.DATA);
    primaryKey.setFieldType(DataSourceFieldType.INTEGER);
    primaryKey.setIsPrimaryKey(true);
    primaryKey.setName("id");
    primaryKey.setWritebackEnabled(false);
    dataSource.getField().add(primaryKey);

    DataSourceField email = new DataSourceField();
    email.setDataType(DataSourceDataType.DATA);
    email.setFieldType(DataSourceFieldType.EMAIL);
    email.setIsPrimaryKey(false);
    email.setName("email");
    email.setWritebackEnabled(false);
    dataSource.getField().add(email);

    DataSourceField phone = new DataSourceField();
    phone.setDataType(DataSourceDataType.DATA);
    phone.setFieldType(DataSourceFieldType.SMS);
    phone.setIsPrimaryKey(false);
    phone.setName("phone");
    phone.setWritebackEnabled(false);
    dataSource.getField().add(phone);

    DataSourceField status = new DataSourceField();
    status.setDataType(DataSourceDataType.STATUS);
    status.setFieldType(DataSourceFieldType.TEXT);
    status.setIsPrimaryKey(false);
    status.setName("email_status");
    status.setWritebackEnabled(false);
    dataSource.getField().add(status);

    DataSourceField unsub = new DataSourceField();
    unsub.setDataType(DataSourceDataType.UNSUBSCRIBE);
    unsub.setFieldType(DataSourceFieldType.TEXT);
    unsub.setIsPrimaryKey(false);
    unsub.setName("unsub");
    unsub.setWritebackEnabled(false);
    dataSource.getField().add(unsub);

    DataSourceField fname = new DataSourceField();
    fname.setDataType(DataSourceDataType.DATA);
    fname.setFieldType(DataSourceFieldType.TEXT);
    fname.setIsPrimaryKey(false);
    fname.setName("first_name");
    fname.setWritebackEnabled(false);
    dataSource.getField().add(fname);

    DataSourceField lname = new DataSourceField();
    lname.setDataType(DataSourceDataType.DATA);
    lname.setFieldType(DataSourceFieldType.TEXT);
    lname.setIsPrimaryKey(false);
    lname.setName("last_name");
    lname.setWritebackEnabled(false);
    dataSource.getField().add(lname);

    return (ExternalDataSourceId)helper.create(dataSource);
  }


  /**
   * Add one record using inline to the datasource created in createIDS().
   *
   * @param dataSourceId the data source ID where records will be added to.
   * @throws Exception any exception
   */
  public void addRecordsInline(DataSourceId dataSourceId) throws Exception
  {
    // Create request
    AddDataSourceRecordsRequest addRecordsRequest = new AddDataSourceRecordsRequest();
    addRecordsRequest.setDataSourceId(dataSourceId);

    // Create records based on the datasource schema created in createIDS()
    DataSourceRecord record1 = new DataSourceRecord();
    NameValuePair idCol = new NameValuePair();
    idCol.setName("id");
    idCol.setValue("1");
    record1.getField().add(idCol);

    NameValuePair emailCol = new NameValuePair();
    emailCol.setName("email_address");
    emailCol.setValue("email_for_cxf@cxf.com");
    record1.getField().add(emailCol);

    NameValuePair nameCol = new NameValuePair();
    nameCol.setName("name");
    nameCol.setValue("cxf_name");
    record1.getField().add(nameCol);

    // Wait for data source status to be idle before adding
    helper.waitForIdle(dataSourceId);

    // Add record to request and make call
    addRecordsRequest.getDataSourceRecord().add(record1);
    service.addRecords(addRecordsRequest);
  }


  /**
   * Add records using a file to the datasource created in createIDS().
   *
   * @param dataSourceId the data source ID where records will be added to.
   * @param dataFile     the file that will be uploaded
   * @throws Exception any exception
   */
  public void addRecordsUsingFile(DataSourceId dataSourceId,
                                  File dataFile) throws Exception
  {
    // Create request
    AddDataSourceRecordsRequest addRecordsRequest = new AddDataSourceRecordsRequest();
    addRecordsRequest.setDataSourceId(dataSourceId);

    addRecordsRequest.setDataSourceId(dataSourceId);
    addRecordsRequest.setContainsFieldNames(true);
    addRecordsRequest.setFieldDelimiter(",");

    // Set data handler to read data from the file
    InputStream is = new FileInputStream(dataFile);
    DataHandler dh = new DataHandler(new AttachmentDataSource("text/plain", is));
    addRecordsRequest.setDataSourceRecordsAttachment(dh);

    helper.waitForIdle((DataSourceId)dataSourceId);

    // Make call
    service.addRecords(addRecordsRequest);
  }

  /**
   * Remove the record added in addRecordsInline().
   *
   * @param dataSourceId the DataSourceId whose records will be removed.
   * @return the number of records that was removed.
   * @throws Exception any exception
   */
  public int removeRecordsInline(DataSourceId dataSourceId) throws Exception
  {
    RemoveDataSourceRecordsRequest request = new RemoveDataSourceRecordsRequest();
    request.setDataSourceId(dataSourceId);
    request.setMatchFieldName("name");
    request.getRecord().add("cxf_name");

    return service.removeRecords(request).getRecordsRemoved();
  }

  /**
   * Export data source records to a file.
   *
   * @param dataSourceId the data source to be exported
   * @param toFile       the file to be written to
   * @throws Exception any exception
   */
  public void exportRecords(DataSourceId dataSourceId, File toFile) throws Exception
  {
    // Create and fill in request
    ExportDataSourceRecordsRequest request = new ExportDataSourceRecordsRequest();
    request.setDataSourceId(dataSourceId);
    request.setFieldDelimiter("|");
    request.setRowDelimiter("\n");

    // Export regular records. Use true to export malformed records.
    request.setUseMalformedRecords(false);

    // Wait for data source status to be idle before exporting
    helper.waitForIdle(dataSourceId);

    // Make call and save data to file
    ExportRecordsResponse response = service.exportRecords(request);
    InputStream is = response.getData().getInputStream();
    OutputStream os = new FileOutputStream(toFile);
    IOUtils.copy(is, os);
  }

  /**
   * Retrieve and print out DataSource statistics.
   *
   * @param dataSourceId the data source ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public void getAndPrintStatistics(DataSourceId dataSourceId)
    throws Exception
  {
    // Create request
    GetDataSourceStatisticsRequest request = new GetDataSourceStatisticsRequest();
    request.setDataSourceId(dataSourceId);

    // Make call
    GetDataSourceStatisticsResponse response = (GetDataSourceStatisticsResponse)service.getStatistics(
      request);
    System.out.println("\nStatistics for data source " + dataSourceId.getId());
    System.out.println("Total records: " + response.getTotalRecords());
    System.out.println("Total invalid records: " + response.getTotalInvalidRecords());
    System.out.println("Total malformed records: " + response.getTotalMalformedRecords());
    System.out.println("Total unsub records: " + response.getTotalUnsubscribedRecords());
  }

  /**
   * Get and print datasource object to standard out.
   *
   * @param dataSourceId the data source ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public void getAndPrint(DataSourceId dataSourceId) throws Exception
  {
    DataSource dataSource = (DataSource)helper.get(dataSourceId);

    // Print data source to standard out
    System.out.println("\nDataSource:");
    System.out.println("DataSourceId: " + dataSource.getObjectId().getId());
    System.out.println("Name: " + dataSource.getName());
    System.out.println("Description: " + dataSource.getDescription());
    System.out.println("CampaignId: " + dataSource.getCampaignId());
    System.out.println("CreatedTime: " + dataSource.getCreatedTime());
    System.out.println("ModifiedTime: " + dataSource.getModifiedTime());
    System.out.println("OperationStatus: " + dataSource.getOperationStatus());
    System.out.println("OrganizationId: " + dataSource.getOrganizationId().getId());
    System.out.println("UserId: " + dataSource.getOwnerId().getId());

    // Print out field definition (data source schema)
    List<DataSourceField> fields = dataSource.getField();
    for (DataSourceField field : fields)
    {
      System.out.println(field.getName() + " " + field.getFieldType() + " " + field.getDataType());
    }

    if (dataSourceId instanceof ExternalDataSourceId)
    {
      ExternalDataSource eds = (ExternalDataSource)dataSource;
      System.out.println("AdvQuery: " + eds.getAdvancedQuery());
      System.out.println("DailyRefreshStartTime: " + eds.getDailyRefresh().getStartTime());
      System.out.println("Tablename: " + eds.getTableName());
      System.out.println("WriteBackTable: " + eds.getWritebackTable());
      ExternalDataSource.ConnectionInfo info = eds.getConnectionInfo();
      System.out.println("Connection: " +
                         info.getDatabaseName() + " " +
                         info.getDatabaseType() + " " +
                         info.getHostname() + " " +
                         info.getPort() + " " +
                         info.getUsername() + " " +
                         info.getPassword() // Server does not return password
      );
    }
  }


  /**
   * Update a data source with new name.
   *
   * @param dataSourceId the data source ID whose name will be updated
   * @param newName      the new name for the data source
   * @throws Exception any exception
   */
  public void updateDataSourceName(DataSourceId dataSourceId, String newName)
    throws Exception
  {
    // Get the data source first
    List<ObjectId> dataSourceIds = new ArrayList<ObjectId>(1);
    dataSourceIds.add(dataSourceId);
    DataSource dataSource = (DataSource)helper.get(dataSourceIds).get(0);

    // Change name and update
    dataSource.setName(newName);
    helper.update(dataSource);
  }

  /**
   * Copy a data source using a new name.
   *
   * @param dataSourceId the data source ID that will be copied
   * @param newName      the name of new data source
   * @return the new InternalDataSource ID
   * @throws Exception any exception
   */
  public InternalDataSourceId copyDataSource(InternalDataSourceId dataSourceId,
                                             String newName) throws Exception
  {
    // Create copy request
    CopyDataSourceRequest request = new CopyDataSourceRequest();
    request.setFromId(dataSourceId);
    request.setNewName(newName);

    return (InternalDataSourceId)service.copy(request).getObjectId();
  }


  /**
   * Refresh External Data Source records.
   *
   * @param edsId the EDS ID to be refreshed.
   * @return if refresh is successful
   * @throws Exception any exception
   */
  public boolean refreshRecords(ExternalDataSourceId edsId) throws Exception
  {
    RefreshRecordsRequest request = new RefreshRecordsRequest();
    request.setDataSourceId(edsId);

    return service.refreshRecords(request).isSuccess();
  }


  public boolean cancelRefreshRecords(ExternalDataSourceId edsId) throws Exception
  {
    CancelRefreshRecordsRequest request = new CancelRefreshRecordsRequest();
    request.setDataSourceId(edsId);

    return service.cancelRefreshRecords(request).isSuccess();
  }

  /**
   * Remove duplicate records for a data source based on field "name". If duplicate records are
   * found, keep the latest ones.
   *
   * @param idsId the InternalDataSourceId that will be deduped
   * @return if dedupe is successful
   * @throws Exception any exception
   */
  public boolean dedupeRecords(InternalDataSourceId idsId) throws Exception
  {
    DedupeDataSourceRecordsRequest request = new DedupeDataSourceRecordsRequest();
    request.setDataSourceId(idsId);
    request.setOption(DataSourceDedupeOption.KEEP_NEWEST);
    request.getMatchField().add("name");

    return service.dedupeRecords(request).isSuccess();
  }

  /**
   * Add one record using inline to the datasource created in createIDS().
   *
   * @param dataSourceId the data source ID where records will be added to.
   * @throws Exception any exception
   */
  private void addSampleRecordForUpsert(DataSourceId dataSourceId) throws Exception
  {
    // Create request
    AddDataSourceRecordsRequest addRecordsRequest = new AddDataSourceRecordsRequest();
    addRecordsRequest.setDataSourceId(dataSourceId);

    // Create records based on the datasource schema created in createIDS()
    DataSourceRecord record1 = new DataSourceRecord();
    for (int i = 1; i < 3; i++)
    {
    NameValuePair idCol = new NameValuePair();
    idCol.setName("id");
    idCol.setValue("90" + i);
    record1.getField().add(idCol);

    NameValuePair emailCol = new NameValuePair();
    emailCol.setName("email_address");
    emailCol.setValue("email_for_cxf_upsert" + i + "@cxf.com");
    record1.getField().add(emailCol);

    NameValuePair nameCol = new NameValuePair();
    nameCol.setName("name");
    nameCol.setValue("cxf_name_for_upsert" + i);
    record1.getField().add(nameCol);
    }

    // Wait for data source status to be idle before adding
    helper.waitForIdle(dataSourceId);

    // Add record to request and make call
    addRecordsRequest.getDataSourceRecord().add(record1);
    service.addRecords(addRecordsRequest);
  }


  public void upsertRecord(DataSourceId dataSourceId) throws Exception
  {
    // A non-existent record, which will get added.
    UpsertDataSourceRecordsRequest upsertRequest1 = new UpsertDataSourceRecordsRequest();
    upsertRequest1.setDataSourceId(dataSourceId);
    DataSourceRecord newRecord = new DataSourceRecord();
    NameValuePair idCol = new NameValuePair();
    idCol.setName("id");
    idCol.setValue("903");
    newRecord.getField().add(idCol);
    NameValuePair emailCol = new NameValuePair();
    emailCol.setName("email_address");
    emailCol.setValue("email_for_cxf_upsert3@cxf.com");
    newRecord.getField().add(emailCol);
    NameValuePair nameCol = new NameValuePair();
    nameCol.setName("name");
    nameCol.setValue("cxf_name_for_upsert3");
    newRecord.getField().add(nameCol);
    upsertRequest1.getDataSourceRecord().add(newRecord);
    // For upsertRecord api we do not need to check IDS status.
    service.upsertRecord(upsertRequest1);

    // An existing record, which will get updated.
    UpsertDataSourceRecordsRequest upsertRequest2 = new UpsertDataSourceRecordsRequest();
    upsertRequest2.setDataSourceId(dataSourceId);
    DataSourceRecord updateRecord = new DataSourceRecord();
    idCol = new NameValuePair();
    idCol.setName("id");
    idCol.setValue("1");
    updateRecord.getField().add(idCol);
    emailCol = new NameValuePair();
    emailCol.setName("email_address");
    emailCol.setValue("email_for_cxf_upsert-updated@cxf.com");
    updateRecord.getField().add(emailCol);
    upsertRequest2.getDataSourceRecord().add(updateRecord);
    // For upsertRecord api we do not need to check IDS status.
    service.upsertRecord(upsertRequest2);
  }

  private void printDataSourceRecord(DataSourceRecord record)
  {
    System.out.println("---------------------------");
    for(NameValuePair field : record.getField())
    {
      System.out.println("Column: " + field.getName() + ", Value: " + field.getValue());
    }
  }

  // Get the upsert-ed records using the getRecords API.
  public void getRecord(DataSourceId dataSourceId) throws Exception
  {
    // Use primary key (id) to fetch the record that was newly added via upsert previously.
    GetDataSourceRecordsRequest getRecordRequest1 = new GetDataSourceRecordsRequest();
    getRecordRequest1.setDataSourceId(dataSourceId);
    NameValuePair idCol = new NameValuePair();
    if(dataSourceId instanceof InternalDataSourceId) {
      idCol.setName("id");
      idCol.setValue("903");
    } else {
      idCol.setName("email_address");
      idCol.setValue("email_for_cxf_upsert3@cxf.com");
    }
    getRecordRequest1.getMatchFields().add(idCol);
    GetDataSourceRecordsResponse getRecordsResponse1 = (GetDataSourceRecordsResponse)service.getRecords(getRecordRequest1);

    if (getRecordsResponse1.getDataSourceRecord() != null && getRecordsResponse1.getDataSourceRecord().size() > 0)
    {
      System.out.println("Records fetched: ");
      for (DataSourceRecord record : getRecordsResponse1.getDataSourceRecord())
      {
        printDataSourceRecord(record);
      }
    }

    // Use the email field to fetch the existing record that was updated via upsert previously.
    GetDataSourceRecordsRequest getRecordRequest2 = new GetDataSourceRecordsRequest();
    getRecordRequest2.setDataSourceId(dataSourceId);
    NameValuePair emailCol = new NameValuePair();
    emailCol.setName("email_address");
    emailCol.setValue("email_for_cxf_upsert-updated@cxf.com");
    getRecordRequest2.getMatchFields().add(emailCol);
    GetDataSourceRecordsResponse getRecordsResponse2 = (GetDataSourceRecordsResponse)service.getRecords(getRecordRequest2);

    if (getRecordsResponse2.getDataSourceRecord() != null && getRecordsResponse2.getDataSourceRecord().size() > 0)
    {
      System.out.println("Records fetched: ");
      for (DataSourceRecord record : getRecordsResponse2.getDataSourceRecord())
      {
        printDataSourceRecord(record);
      }
    }
  }

  /**
   * Create an Internal Data Source with 3 columns: id, email_address, and name.
   *
   * @param dsname the name of Internal Data Source
   * @return the ID of Internal Data Source
   * @throws Exception any exception
   */
  public InternalDataSourceExtnId createIDX(String dsname)
    throws Exception
  {
    // Create InternalDataSource and fill in properties.
    InternalDataSourceExtn dataSource = new InternalDataSourceExtn();
    dataSource.setDescription("Internal Data Source from CXF");
    dataSource.setName(dsname);

    // Specify the data source fields: id (primary key), email and name.
    // System will generate "unsub" and "email_status" fields
    DataSourceField primaryKey = new DataSourceField();
    primaryKey.setDataType(DataSourceDataType.DATA);
    primaryKey.setFieldType(DataSourceFieldType.INTEGER);
    primaryKey.setIsPrimaryKey(false);
    primaryKey.setName("id");
    primaryKey.setWritebackEnabled(false);
    dataSource.getField().add(primaryKey);

    DataSourceField email = new DataSourceField();
    email.setDataType(DataSourceDataType.DATA);
    email.setFieldType(DataSourceFieldType.EMAIL);
    email.setIsPrimaryKey(true);
    email.setName("email_address");
    email.setWritebackEnabled(false);
    dataSource.getField().add(email);

    DataSourceField name = new DataSourceField();
    name.setDataType(DataSourceDataType.DATA);
    name.setFieldType(DataSourceFieldType.TEXT);
    name.setIsPrimaryKey(false);
    name.setName("name");
    name.setWritebackEnabled(false);
    dataSource.getField().add(name);

    // Create data source
    InternalDataSourceExtnId dataSourceId = (InternalDataSourceExtnId)helper.create(dataSource);

    // Add some data
    addRecordsInline(dataSourceId);

    return dataSourceId;
  }
}
