/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2013 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CharSet;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlockId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Encoding;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ImportMessagePartRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MediaServerId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MessageFormat;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MessagePart;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NameValuePair;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Template;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TestTemplateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ValidateXslRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NamedLink;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksTemplateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinkResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TrackingTag;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import javax.activation.DataHandler;
import javax.mail.util.ByteArrayDataSource;


public class TemplateSample
{
  private MailingService service;
  private SampleHelper helper;

  public TemplateSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper(service);
  }


  /**
   * Create a template.
   *
   * @param templateName   the name of the template
   * @param attachmentId   the attachment used for this template
   * @param contentBlockId the content block used for this template
   * @return TemplateId
   * @throws Exception any exception
   */
  public TemplateId createTemplate(String templateName,
                                   AttachmentId attachmentId,
                                   ContentBlockId contentBlockId) throws Exception
  {
    Template template = new Template();

    template.setBodyEncoding(Encoding.BASE_64);
    template.setDescription("Template from CXF");
    template.setFromName("From Name");
    template.setHeaderEncoding(Encoding.EIGHT_BIT);
    template.setIsApproved(true);
    template.setName(templateName);
    template.setOutputBodyCharSet(CharSet.ASCII);
    template.setOutputHeaderCharSet(CharSet.UTF_8);
    template.setSubject("CXF subject");

    if(attachmentId != null)
    {
        template.getAttachmentId().add(attachmentId);
    }
    if(contentBlockId !=null)
    {
        template.getContentBlockId().add(contentBlockId);
    }

    template.getHeader().add("CXF template header");

    // Create a message part
    MessagePart messagePart = new MessagePart();
    String content = "<html>" +
                     "<body>" +
                     "Hello, This is a simple HTML message with two links<br/>" +
                     "This is <a href=\"http://www.somedomain.com/link1\">Link 1</a>" +
                     "<br/>This is <a href=\"http://www.somedomain.com/link2\">Link 2</a>" +
                     "</body>" +
                     "</html>"; 
    messagePart.setContent(content);
    messagePart.setFormat(MessageFormat.HTML);
    messagePart.setIsXsl(false);

    template.getMessagePart().add(messagePart);

    // Make a call to fetchLinks.
    // This will return the list of urls found in the message part, encapsulated as named link objects.
    List<NamedLink> links = this.fetchLinks(template, MessageFormat.HTML);

    // By default, each a CLICKTAG will be assigned to each url. The assigment can be changed as follows.
    Map<String, TrackingTag> urlToTagMap = new HashMap<String, TrackingTag>();
    urlToTagMap.put("http://www.somedomain.com/link1", TrackingTag.ENCLICKTAG);
    urlToTagMap.put("http://www.somedomain.com/link2", TrackingTag.UNSUBSCRIBETAG);

    for (int i = 0 ; i < links.size() ; i ++)
    {
      if (urlToTagMap.get(links.get(i).getUrl()) != null)
      {
        links.get(i).setTrackingTag(urlToTagMap.get(links.get(i).getUrl()));
        links.get(i).setName("link" + (i+1));
      }
      // Other properties like web analytics, link name etc can also be set in the named link object. 
    }

    // Assign the updated links to the message part.  
    template.getMessagePart().get(0).getNamedLinks().clear();
    template.getMessagePart().get(0).getNamedLinks().addAll(links);

    // Make call
    return (TemplateId)helper.create(template);
  }

  public void getAndPrint(TemplateId templateId) throws Exception
  {
    Template template = (Template)helper.get(templateId);

    // Print template to standard out
    System.out.println("\nTemplate:");
    System.out.println("TemplateId: " + template.getObjectId().getId());
    System.out.println("Name: " + template.getName());
    System.out.println("CampaignId: " + template.getCampaignId());
    System.out.println("CreatedTime: " + template.getCreatedTime());
    System.out.println("Description: " + template.getDescription());
    System.out.println("ModifiedTime: " + template.getModifiedTime());
    System.out.println("OrganizationId: " + template.getOrganizationId().getId());
    System.out.println("OwnerId: " + template.getOwnerId().getId());
    System.out.println("AttachmentId: " + SampleHelper.formatIdList(template.getAttachmentId()));
    System.out.println("ContentBlockId: " + SampleHelper.formatIdList(template.getContentBlockId()));
    System.out.println("Headers: " + template.getHeader());
    System.out.println("BodyEncoding: " + template.getBodyEncoding().value());
    System.out.println("HeaderEncoding: " + template.getHeaderEncoding().value());
    System.out.println("MessagePart: " + SampleHelper.formatMessagePartList(template.getMessagePart()));
    System.out.println("BodyCharset: " + template.getOutputBodyCharSet().value());
    System.out.println("BodyOutputToken: " + template.getOutputBodyCharSetToken());
    System.out.println("HeaderCharset: " + template.getOutputHeaderCharSet().value());
    System.out.println("HeaderOutputToken: " + template.getOutputHeaderCharSetToken());
    System.out.println("BounceAddressId: " + template.getBounceAddressId());
    System.out.println("FromAddressId: " + template.getFromAddressId());
    System.out.println("FromName: " + template.getFromName());
    System.out.println("ReplyAddressId: " + template.getReplyAddressId());
    System.out.println("Subject: " + template.getSubject());
    System.out.println("Named Links: " + getPrintableNamedLinks(template));
  }


  /**
   * Import message part using a zip file and media server.
   *
   * @param templateId    the template that message part will be added to
   * @param mediaServerId the media server that is used to store the image files.
   * @param zipFile       the zip file that contains HTML and image files.
   * @param folderName    the directory to unzip the zip file
   * @return if the importing operation is successful
   * @throws Exception any exception
   */
  public boolean importMessagePart(TemplateId templateId,
                                   MediaServerId mediaServerId,
                                   File zipFile,
                                   String folderName) throws Exception
  {

    // Create request
    ImportMessagePartRequest request = new ImportMessagePartRequest();
    request.setFolderName(folderName);
    request.setIsXsl(false);
    request.setMediaServerId(mediaServerId);
    request.setTemplateId(templateId);

    // Set zip file
    InputStream is = new FileInputStream(zipFile);
    DataHandler dh = new DataHandler(new ByteArrayDataSource(is, "application/octet-stream"));
    request.setZipFile(dh);

    return service.importMessagePart(request).isSuccess();
  }


  /**
   * Test template for it dynamic content. Email will be sent and tokens in the template will be
   * replaced with values.
   *
   * @param templateId the TemplateId that will be sent out.
   * @return if the test request is sent successfully.
   * @throws Exception any exception
   */
  public boolean testTemplate(TemplateId templateId) throws Exception
  {
    TestTemplateRequest request = new TestTemplateRequest();
    request.setSubjectPrefix("Subject Prefix: ");
    request.setTemplateId(templateId);
    request.setUseMultiPart(false);
    request.getAddress().add("test_address@testtemplate.com");
    request.getFormat().add(MessageFormat.HTML);

    // Make sure you have these two tokens in your template
    NameValuePair pair1 = new NameValuePair();
    pair1.setName("EMAIL");
    pair1.setValue("testemail@testtemplate.com");
    request.getTokenValue().add(pair1);

    NameValuePair pair2 = new NameValuePair();
    pair2.setName("NAME");
    pair2.setValue("testemail name");
    request.getTokenValue().add(pair2);

    return service.test(request).isSuccess();
  }


  /**
   * Validate if a template's xsl content is valid.
   *
   * @param templateId the TemplateId whose XSL content will be validated.
   * @return if the content is valid XSL
   * @throws Exception any exception
   */
  public boolean validateXsl(TemplateId templateId) throws Exception
  {
    ValidateXslRequest request = new ValidateXslRequest();
    request.setMessageFormat(MessageFormat.HTML);
    request.setTemplateId(templateId);

    return service.validateXsl(request).isValid();
  }


  private List<NamedLink> fetchLinks(Template template, MessageFormat format) throws Exception
  {
    List<NamedLink> links = new ArrayList<NamedLink>();
    FetchLinksTemplateRequest request = new FetchLinksTemplateRequest();
    request.setTemplate(template);
    request.setMessageFormat(format);
    FetchLinksResponse response = service.fetchLinks(request);
    List<FetchLinkResponse> individualLinkResponse = response.getFetchLinkResponse();
    if (individualLinkResponse != null && individualLinkResponse.size() > 0)
    {
      for (FetchLinkResponse individualLink : individualLinkResponse)
      {
        links.add(individualLink.getNamedLink());
      }
    }

    return links;
  }

  private String getPrintableNamedLinks(Template template)
  {
    StringBuffer printableNamedLinks = new StringBuffer("[");
    List<MessagePart> messageParts = template.getMessagePart();
    if (messageParts != null)
    {
      for (int p = 0; p < messageParts.size(); p++)
      {
        if (messageParts.get(p).getNamedLinks() != null && messageParts.get(p).getNamedLinks().size() > 0)
        {
          if (p > 0)
          {
            printableNamedLinks.append(", ");
          }
          printableNamedLinks.append(" [" + messageParts.get(p).getFormat().name() + " - ");
          List<NamedLink> links = messageParts.get(p).getNamedLinks();
          for (int n = 0; n < links.size(); n++)
          {
            if (n > 0)
            {
              printableNamedLinks.append(", ");
            }
            printableNamedLinks.append(links.get(n).getName() + ";" +
                                       links.get(n).getUrl() + ";" +
                                       links.get(n).getTrackingTag().name());
          }

          printableNamedLinks.append("]");
        }

      }
    }

    printableNamedLinks.append("]");

    return printableNamedLinks.toString();
  }
}
