<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


require_once "GlobalVariables.php";
require_once "./generated/MailingService.php";

class SampleHelper
{
  private $service;

  public function __construct($service)
  {
    $this->service = $service;
  }

  /**
   * Get a list of objects based on given IDs.
   *
   * @param objectIds list of IDs that user wants to get
   * @return list of BaseObjects
   * @throws Exception any exception
   */
  public function get($objectIds)
  {
    // get request and set objectIds
    $request = new GetRequest();
    $request->objectId = $objectIds;

    // Create list of BaseObject
    $result = array();

    // Make call
    $batchGetResponse = $this->service->get($request);

    // Go through each response inside batch response and get the object if it is successful
    // Print out error message if any object is not retrieved successfully
    // and set the object to null.
    $getResponses = $batchGetResponse->getResponse;
    $successes = $batchGetResponse->success;
    $faults = $batchGetResponse->fault;
    for ($i = 0; $i < count($objectIds); $i++)
    {
      if ($successes[$i])
      {
        $result[] = $getResponses[$i]->baseObject;
      }
      else
      {
        $fault = $faults[$i];
        print($fault->faultCode . ": " . $fault->faultMessage . "\n");
        $result[] = null;
      }
    }

    return $result;
  }

  /**
   * Get a list of objects based on given IDs.
   *
   * @param object (asset) which user wants to create
   * @return objectId of created asset
   * @throws Exception any exception
   */
  public function create($object)
  {
    // create request and set objectIds
    $typeName = get_class($object);
    print("\nCreating:". $typeName . "...");
    $request = SampleHelper::getSoapCreateRequestObject($object,$typeName);

    // Make call
    $batchCreateResponse = $this->service->create($request);

    // Go through each response inside batch response and get the object if it is successful
    // Print out error message if fault occurred during create operation
    $createResponses = $batchCreateResponse->createResponse;
    $successes = $batchCreateResponse->success;
    $faults = $batchCreateResponse->fault;

    if (!$successes[0])
    {
      $fault = $faults[0];
      print($fault->faultCode . ": " . $fault->faultMessage . "\n");
    }
    GlobalVariables::$object_ids[] = $createResponses[0]->objectId;
    return $createResponses[0];
  }

  /**
   * Delete a list of objects.
   *
   * @param objectIds the list of objectIds that will be deleted.
   * @throws Exception any exception
   */
  public function delete()
  {
    // Create delete request
    $request = new DeleteRequest();
    $request->objectId = GlobalVariables::$object_ids;

    // Make call
    $this->service->delete($request);
  }

  static function getSchemaVer()
  {
    return GlobalVariables::$schema_version;
  }

  static function getMailingService()
  {
    $options['trace'] = true;
    $options['exceptions'] = true;
    $options['features'] = SOAP_SINGLE_ELEMENT_ARRAYS;

    return new MailingService(GlobalVariables::$wsdl_url, $options);
  }

  static function getSoapCreateRequestObject($data, $typeName)
  {
    $request = new CreateRequest();
    $request->baseObject = SampleHelper::getSoapRequestVariable($data, $typeName);

    return $request;
  }

  static function getSoapUpdateRequestObject($data, $typeName)
  {
    $request = new UpdateRequest();
    $request->baseObject = SampleHelper::getSoapRequestVariable($data, $typeName);

    return $request;
  }

  static function getSoapGetRequestObject($entityIdObject, $idValue, $typeName)
  {
    $request = new GetRequest();
    $request->objectId = SampleHelper::getSoapEntityIdObject($entityIdObject,
                                                             $idValue,
                                                             $typeName);

    return $request;
  }

  static function getSoapListRequestObject($filterObj, $typeName)
  {
    $request = new ListRequest();
    $request->filter = SampleHelper::getSoapRequestVariable($filterObj, $typeName);

    return $request;
  }

  static function getSoapEntityIdObject($entityIdObject, $idValue, $typeName)
  {
    $entityIdObject->id = $idValue;
    return SampleHelper::getSoapRequestVariable($entityIdObject, $typeName);
  }

  static function getSoapRequestVariable($data, $typeName)
  {
    return new SoapVar($data, SOAP_ENC_OBJECT, $typeName, SampleHelper::getSchemaVer(), NULL, NULL);
  }

  static function formatIdList($objectIds)
  {
    $buffer = "[";
    for ($i=0; $i<count($objectIds); $i++)
    {
      $buffer = $buffer . $objectIds[$i]->id;
      if ($i < count($objectIds)-1)
      {
        $buffer = $buffer . ", ";
      }
    }
    $buffer = $buffer . "]";

    return $buffer;
  }

  static function formatMessagePartList($messageParts)
  {
    $buffer = "[";
    for ($i=0; $i<count($messageParts); $i++)
    {
      $buffer = $buffer . "{" . $messageParts[$i]->format . ":" . $messageParts[$i]->content . "}";
      if ($i < count($messageParts)-1)
      {
        $buffer = $buffer . ", ";
      }
    }
    $buffer = $buffer . "]";

    return $buffer;
  }

  static function formatStringList($list)
  {
    $buffer = "[";
    for ($i=0; $i<count($list); $i++)
    {
      $buffer = $buffer . $list[$i];
      if ($i < count($list)-1)
      {
        $buffer = $buffer . ", ";
      }
    }
    $buffer = $buffer . "]";

    return $buffer;
  }

  // Wait for one minute for data source operation status to be IDLE
  // Database operation (add/remove records) is asynchronous
  public function waitForIdle($dataSourceId)
  {
    $end = time() + 60; // up to one minute

    while (time() < $end)
    {
      try
      {
        $datasource = $this->get($dataSourceId);
        if ($datasource[0]->operationStatus != "IDLE")
        {
          sleep(2);
        }
        else
        {
          break;
        }
      }
      catch (Exception $e)
      {
        // ok
      }
    }
  }
}
GlobalVariables::$object_ids = array();

?>
