/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlock;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlockId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DeepLink;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NamedLink;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinkResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksContentBlockRequest;

import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;


public class ContentBlockSample
{
  private MailingService service;
  private SampleHelper helper;

  public ContentBlockSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper();
  }

  /**
   * Create a content block.
   *
   * @param contentBlockName the name of the content block
   * @return contentBlockId
   * @throws Exception any exception
   */
  public ContentBlockId createContentBlock(String contentBlockName) throws Exception
  {
    ContentBlock contentBlock = new ContentBlock();

    contentBlock.setDescription("ContentBlock from CXF");
    contentBlock.setName(contentBlockName);

    String content = "hiking {\n" +
                     "  I love hiking.\n" +
                     "}\n" +
                     "scuba {\n" +
                     "  I love SCUBA diving.\n" +
                     "}\n" +
                     "tracking_tag_sample {\n" +
                     "Hello, This is a sample of tracking tags in content block<br/>" +
                     "This is <a href=\"http://www.somedomain.com/link1\">Link 1</a>" +
                     "<br/>This is <a href=\"http://www.somedomain.com/link2\">Link 2</a>" +
                     "}\n";
    contentBlock.setContent(content);

    // Make a call to fetchLinks.
    // This will return the list of urls found in the message part, encapsulated as named link objects.
    List<NamedLink> links = this.fetchLinks(contentBlock);

    // By default, each a CLICKTAG will be assigned to each url. The assigment can be changed as follows.
    Map<String, String> urlToTagMap = new HashMap<String, String>();
    urlToTagMap.put("http://www.somedomain.com/link1", "CLICK");
    urlToTagMap.put("http://www.somedomain.com/link2", "UNSUB");

    for (int i = 0 ; i < links.size() ; i ++)
    {
      if (urlToTagMap.get(links.get(i).getUrl()) != null)
      {
        links.get(i).setTagType(urlToTagMap.get(links.get(i).getUrl()));
        links.get(i).setTagFormat("FULL");
        links.get(i).setName("link" + (i+1));
      }
      // Other properties like web analytics, deepLink, link name etc can also be set in the named link object.
    }

    // Assign the updated links to the message part.
    contentBlock.getNamedLinks().clear();
    contentBlock.getNamedLinks().addAll(links);

    // Make call
    return (ContentBlockId)helper.create(contentBlock);
  }

  /**
   * Create a content block with deep links.
   *
   * @param contentBlockName the name of the content block
   * @return contentBlockId
   * @throws Exception any exception
   */
  public ContentBlockId createContentBlockWithDeepLinks(String contentBlockName) throws Exception
  {
    ContentBlock contentBlock = new ContentBlock();

    contentBlock.setDescription("ContentBlock from CXF");
    contentBlock.setName(contentBlockName);

    String content = "hiking {\n" +
                     "  I love hiking.\n" +
                     "}\n" +
                     "scuba {\n" +
                     "  I love SCUBA diving.\n" +
                     "}\n" +
                     "tracking_tag_sample {\n" +
                     "Hello, This is a sample of tracking tags in content block<br/>" +
                     "This is <a href=\"http://www.somedomain.com/link1\">Link 1</a>" +
                     "<br/>This is <a href=\"http://www.somedomain.com/link2\">Link 2</a>" +
                     "}\n";
    contentBlock.setContent(content);

    // Make a call to fetchLinks.
    // This will return the list of urls found in the message part, encapsulated as named link objects.
    List<NamedLink> links = this.fetchLinks(contentBlock);

    // By default, each a CLICKTAG will be assigned to each url. The assigment can be changed as follows.
    Map<String, String> urlToTagMap = new HashMap<String, String>();
    urlToTagMap.put("http://www.somedomain.com/link1", "CLICK");
    urlToTagMap.put("http://www.somedomain.com/link2", "UNSUB");

    for (int i = 0 ; i < links.size() ; i ++)
    {
      if (urlToTagMap.get(links.get(i).getUrl()) != null)
      {
        links.get(i).setTagType(urlToTagMap.get(links.get(i).getUrl()));
        links.get(i).setTagFormat("Obfuscated");
        // Obfuscated
        links.get(i).setName("link" + (i+1));
      }
      // Other properties like web analytics, deepLink, link name etc can also be set in the named link object.
      DeepLink deepLink1 = new DeepLink();
      deepLink1.setType("IOS_App");
      deepLink1.setUrl("www.CBIOSAppTestUrl.com");

      DeepLink deepLink2 = new DeepLink();
      deepLink2.setType("IOS_Fallback");
      deepLink2.setUrl("www.CBIOSFallbackTestUrl.com");

      DeepLink deepLink3 = new DeepLink();
      deepLink3.setType("Android_App");
      deepLink3.setUrl("www.CBAndroidAppTestUrl.com");

      DeepLink deepLink4 = new DeepLink();
      deepLink4.setType("Android_Fallback");
      deepLink4.setUrl("www.CBAndroidFallbackTestUrl.com");

      links.get(i).getDeepLinks().add(deepLink1);
      links.get(i).getDeepLinks().add(deepLink2);
      links.get(i).getDeepLinks().add(deepLink3);
      links.get(i).getDeepLinks().add(deepLink4);
    }

    // Assign the updated links to the message part.
    contentBlock.getNamedLinks().clear();
    contentBlock.getNamedLinks().addAll(links);

    // Make call
    return (ContentBlockId)helper.create(contentBlock);
  }

  public void getAndPrint(ContentBlockId contentBlockId) throws Exception
  {
    ContentBlock contentBlock = (ContentBlock)helper.get(contentBlockId);

    // Print ContentBlock to standard out
    System.out.println("\nContentBlock:");
    System.out.println("ContentBlockId: " + contentBlock.getObjectId().getId());
    System.out.println("Name: " + contentBlock.getName());
    System.out.println("CampaignId: " + contentBlock.getCampaignId());
    System.out.println("CreatedTime: " + contentBlock.getCreatedTime());
    System.out.println("Description: " + contentBlock.getDescription());
    System.out.println("ModifiedTime: " + contentBlock.getModifiedTime());
    System.out.println("OrganizationId: " + contentBlock.getOrganizationId().getId());
    System.out.println("OwnerId: " + contentBlock.getOwnerId().getId());
    System.out.println("Content: " + contentBlock.getContent());
    System.out.println("Size: " + contentBlock.getSize());
    System.out.println("Named Links: " + getPrintableNamedLinks(contentBlock));
  }

  private List<NamedLink> fetchLinks(ContentBlock contentBlock) throws Exception
  {
    List<NamedLink> links = new ArrayList<NamedLink>();
    FetchLinksContentBlockRequest request = new FetchLinksContentBlockRequest();
    request.setContentBlock(contentBlock);
    FetchLinksResponse response = service.fetchLinks(request);
    List<FetchLinkResponse> individualLinkResponse = response.getFetchLinkResponse();
    if (individualLinkResponse != null && individualLinkResponse.size() > 0)
    {
      for (FetchLinkResponse individualLink : individualLinkResponse)
      {
        links.add(individualLink.getNamedLink());
      }
    }

    return links;
  }

  private String getPrintableNamedLinks(ContentBlock contentBlock)
  {
    StringBuffer printableNamedLinks = new StringBuffer("[");
    if (contentBlock.getNamedLinks() != null && contentBlock.getNamedLinks().size() > 0)
    {
      List<NamedLink> links = contentBlock.getNamedLinks();
      for (int n = 0; n < links.size(); n++)
      {
        if (n > 0)
        {
          printableNamedLinks.append(", ");
        }
        printableNamedLinks.append(links.get(n).getName() + ";" +
                                   links.get(n).getUrl() + ";" +
                                   links.get(n).getTagType());
      }
    }
    printableNamedLinks.append("]");
    return printableNamedLinks.toString();
  }
}
