/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ColumnRuleValue;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ComparisonOperation;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.LogicalOperation;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Rule;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleElsePart;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleIfPart;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleIfPartCondition;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleThenPart;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Target;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TextRuleValue;


public class RuleSample
{
  private MailingService service;
  private SampleHelper helper;

  public RuleSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper();
  }


  /**
   * Create a rule using the Target created in TargetSample.
   *
   * @param ruleName the name of the rule to be created
   * @param targetId the targetId to be used with this rule.
   * @return ruleId
   * @throws Exception any exception
   */
  public RuleId createRule(String ruleName, TargetId targetId) throws Exception
  {
    Rule rule = new Rule();

    rule.setDescription("Rule from CXF");
    rule.setName(ruleName);
    rule.getTargetId().add(targetId);

    // Create two conditions using the target created in TargetSample
    Target target = (Target)helper.get(targetId);
    RuleIfPartCondition condition1 = new RuleIfPartCondition();
    condition1.setColumn(target.getName() + ".email_address");
    condition1.setOp(ComparisonOperation.EQUALS);
    condition1.setValue("email_for_cxf@cxf.com");

    RuleIfPartCondition condition2 = new RuleIfPartCondition();
    condition2.setColumn(target.getName() + ".name");
    condition2.setOp(ComparisonOperation.CONTAINS);
    condition2.setValue("cxf");

    // Set If part, format for conditions are: Condition1 AND/OR Condition AND/OR Condition ...
    // The above two conditions will return true
    RuleIfPart ifpart = new RuleIfPart();
    ifpart.setCondition1(condition1); // Set the first condition
    ifpart.getLogicalOperationAndCondition().add(LogicalOperation.AND); // Set logical operator
    ifpart.getLogicalOperationAndCondition().add(condition2);   // Set another condition
    rule.setIfPart(ifpart);

    // Set Then part
    TextRuleValue thenvalue = new TextRuleValue();
    thenvalue.setValue("This is then part");
    RuleThenPart thenpart = new RuleThenPart();
    thenpart.setTextRuleValue(thenvalue);
    rule.setThenPart(thenpart);

    // Set Else part
    ColumnRuleValue elsevalue = new ColumnRuleValue();
    elsevalue.setValue(target.getName() + ".name");
    RuleElsePart elsepart = new RuleElsePart();
    elsepart.setColumnRuleValue(elsevalue);
    rule.setElsePart(elsepart);

    // Make call
    return (RuleId)helper.create(rule);
  }


  public void getAndPrint(RuleId ruleId) throws Exception
  {
    Rule rule = (Rule)helper.get(ruleId);

    // Print Rule to standard out
    System.out.println("\nRule:");
    System.out.println("RuleId: " + rule.getObjectId().getId());
    System.out.println("Name: " + rule.getName());
    System.out.println("TargetId: " + SampleHelper.formatIdList(rule.getTargetId()));
    System.out.println("CampaignId: " + rule.getCampaignId());
    System.out.println("CreatedTime: " + rule.getCreatedTime());
    System.out.println("Description: " + rule.getDescription());
    System.out.println("ModifiedTime: " + rule.getModifiedTime());
    System.out.println("OrganizationId: " + rule.getOrganizationId().getId());
    System.out.println("OwnerId: " + rule.getOwnerId().getId());

    // Print condition 1
    RuleIfPartCondition condition1 = rule.getIfPart().getCondition1();
    System.out.println("Rule-If:");
    System.out.println("\t" + condition1.getColumn() + " " + condition1.getOp() + " " + condition1.getValue());

    // Print condition 2
    LogicalOperation logicalOp = (LogicalOperation)rule.getIfPart().getLogicalOperationAndCondition().get(
      0);
    RuleIfPartCondition condition2 = (RuleIfPartCondition)rule.getIfPart().getLogicalOperationAndCondition().get(
      1);
    System.out.println("\t" + logicalOp + " " + condition2.getColumn() + " " + condition2.getOp() + " " + condition2.getValue());

    // Print Then part: use text value since it was set above
    System.out.println("Rule-Then: " + rule.getThenPart().getTextRuleValue().getValue());

    // Print Else part: use column value since it was set above
    System.out.println("Rule-Else: " + rule.getElsePart().getColumnRuleValue().getValue());
  }
}
