/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.BaseFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.BaseObject;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.BaseTemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.BatchCreateResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.BatchGetResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlockId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CreateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSource;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DeleteRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FaultDetail;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MessagePart;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SeedListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SuppressionListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddress;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.UpdateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.User;

import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;


public class SampleHelper
{
  private static MailingService service;

  static
  {
    service = createService();
  }

  public static boolean isLogRequests()
  {
    return logRequests;
  }

  public static void setLogRequests(boolean logRequests)
  {
    SampleHelper.logRequests = logRequests;
  }

  private static boolean logRequests;

  /**
   * Get one object based on its ID.
   *
   * @param objectId the objectId to retrieve
   * @return an object whose ID is objectId
   * @throws Exception any exception
   */
  public BaseObject get(ObjectId objectId) throws Exception
  {
    List<ObjectId> objectIds = new ArrayList<ObjectId>();
    objectIds.add(objectId);

    return get(objectIds).get(0);
  }

  /**
   * Get a list of objects based on given IDs.
   *
   * @param objectIds list of IDs that user wants to get
   * @return list of BaseObjects
   * @throws Exception any exception
   */
  public List<BaseObject> get(List<? extends ObjectId> objectIds)
    throws Exception
  {

    // Create request and set objectIds
    GetRequest request = new GetRequest();
    request.getObjectId().addAll(objectIds);

    // Create list of BaseObject with the same size as list of IDs
    List<BaseObject> result = new ArrayList<BaseObject>(objectIds.size());

    // Make call
    BatchGetResponse batchGetResponse = service.get(request);

    // Go through each response inside batch response and get the object if it is successful
    // Print out error message if any object is not retrieved successfully
    // and set the object to null.
    for (int i = 0; i < objectIds.size(); i++)
    {
      if (batchGetResponse.getSuccess().get(i))
      {
        result.add(batchGetResponse.getGetResponse().get(i).getBaseObject());
      }
      else
      {
        FaultDetail faultDetail = batchGetResponse.getFault().get(i);
        System.out.println(faultDetail.getFaultCode() + ": " + faultDetail.getFaultMessage());
        result.add(null);
      }
    }

    return result;
  }


  /**
   * Create an object.
   *
   * @param anObject the object that will be created.
   * @return the objectId
   * @throws Exception any exception
   */
  public ObjectId create(BaseObject anObject) throws Exception
  {
    List<BaseObject> objects = new ArrayList<BaseObject>();
    objects.add(anObject);
    return create(objects).get(0);
  }


  /**
   * Create a list of objects.
   *
   * @param objects list objects that will be created
   * @return list of objectIds for the created objects
   * @throws Exception any exception
   */
  public List<ObjectId> create(List<BaseObject> objects)
    throws Exception
  {
    // Create request and add all objects to be created
    CreateRequest request = new CreateRequest();
    request.getBaseObject().addAll(objects);

    // Create a list to hold created ObjectIds
    List<ObjectId> result = new ArrayList<ObjectId>(objects.size());

    // Make call
    BatchCreateResponse batchCreateResponse = service.create(request);

    // Go through each response inside batch response and get the objectId if it is successful
    // Print out error message if any object is not created successfully
    // and set the objectId to null.
    for (int i = 0; i < objects.size(); i++)
    {
      if (batchCreateResponse.getSuccess().get(i))
      {
        ObjectId objectId = batchCreateResponse.getCreateResponse().get(i).getObjectId();
        result.add(objectId);
        getCreatedObjects().add(objectId);
        getAndPrintData(objectId);
      }
      else
      {
        FaultDetail faultDetail = batchCreateResponse.getFault().get(i);
        String objectName = getNameFromBaseObject(objects.get(i));
        System.out.println(objectName + ":" + faultDetail.getFaultCode() + ": " + faultDetail.getFaultMessage());
        result.add(null);
      }
    }

    return result;
  }


  /**
   * Update an object.
   *
   * @param anObject an object that will be updated.
   */
  public void update(BaseObject anObject)
  {
    // Create update request
    UpdateRequest request = new UpdateRequest();
    request.getBaseObject().add(anObject);

    // Make call
    service.update(request);
  }


  /**
   * Delete an object
   *
   * @param objectId the objectId that will be deleted
   * @throws Exception any exception
   */
  public void delete(ObjectId objectId) throws Exception
  {
    List<ObjectId> objectIds = new ArrayList<ObjectId>();
    objectIds.add(objectId);

    delete(objectIds);
  }


  /**
   * Delete a list of objects.
   *
   * @param objectIds the list of objectIds that will be deleted.
   * @throws Exception any exception
   */
  public void delete(List<ObjectId> objectIds) throws Exception
  {
    // Create delete request
    DeleteRequest request = new DeleteRequest();
    request.getObjectId().addAll(objectIds);

    // Make call
    service.delete(request);
  }

  public static void deleleGeneratedAssests() throws Exception
  {
    new SampleHelper().delete(objectIds);
  }


  /**
   * Get name property of an object by reflection. SystemAddress and User have no name property. Use
   * email address or login as name.
   *
   * @param object an object whose name will be retrieved
   * @return the name property of the object
   */
  private static String getNameFromBaseObject(BaseObject object)
  {
    String name = null;

    try
    {
      Method getNameMethod = object.getClass().getMethod("getName");
      if (getNameMethod != null)
      {
        name = (String)getNameMethod.invoke(object);
      }
    }
    catch (Exception e)
    {
      // SystemAddress and User do not have name
      if (object instanceof SystemAddress)
      {
        return ((SystemAddress)object).getEmailAddress();
      }
      else if (object instanceof User)
      {
        return ((User)object).getLogin();
      }
      else
      {
        // This should not happen.
        e.printStackTrace();
        return null;
      }
    }

    return name;
  }


  /**
   * Set values to a filter: get the first page, 10 records per page, and using ascending order.
   *
   * @param filter a BaseFilter whose attributes will be set
   */
  public void setFilter(BaseFilter filter)
  {
    filter.setIsAscending(true);
    filter.setPageNumber(0);
    filter.setRecordsPerPage(10);
    //filter.setMaxRecordsPerPage();  // Set by server
  }

  public static String formatIdList(List<? extends ObjectId> objectIds)
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append("[");
    for (int i = 0; i < objectIds.size(); i++)
    {
      ObjectId objectId = objectIds.get(i);

      buffer.append(objectId.getId());
      if (i < objectIds.size() - 1)
      {
        buffer.append(", ");
      }
    }
    buffer.append("]");

    return buffer.toString();
  }

  public static String formatMessagePartList(List<MessagePart> messageParts)
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append("[");
    for (int i = 0; i < messageParts.size(); i++)
    {
      MessagePart messagePart = messageParts.get(i);

      buffer = buffer.append("{")
        .append(messagePart.getFormat()).append(":").append(messagePart.getContent())
        .append("}");

      if (i < messageParts.size() - 1)
      {
        buffer.append(", ");
      }
    }
    buffer.append("]");

    return buffer.toString();
  }

  public static String formatList(List list)
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append("[");
    for (int i = 0; i < list.size(); i++)
    {
      Object obj = list.get(i);

      buffer.append(obj.toString());
      if (i < list.size() - 1)
      {
        buffer.append(", ");
      }
    }
    buffer.append("]");

    return buffer.toString();
  }

  // Wait for one minute for data source operation status to be IDLE
  // Database operation (add/remove records) is asynchronous
  public void waitForIdle(DataSourceId dataSourceId) throws Exception
  {
    long end = System.currentTimeMillis() + 60000; // up to one minute
    DataSource datasource;

    while (System.currentTimeMillis() < end)
    {
      try
      {
        datasource = (DataSource)get(dataSourceId);
        if (!datasource.getOperationStatus().equals("IDLE"))
        {
          Thread.sleep(1000);
        }
        else
        {
          break;
        }
      }
      catch (InterruptedException e)
      {
        // ok
      }
    }
  }

  public static MailingService createService()
  {
    // Read key store, key store password, wsdl file location and endpoint address
    // Endpoint address will replace the one in WSDL file
      System.out.println("Store: " + System.getProperty("javax.net.ssl.trustStore"));
    System.out.println("Password: " + System.getProperty("javax.net.ssl.trustStorePassword"));
    System.out.println("WSDL: " + System.getProperty("wsdl"));
    System.out.println("endpoint: " + System.getProperty("endpoint.address"));

    File wsdlFile = new File(System.getProperty("wsdl"));
    String endpoint = System.getProperty("endpoint.address");

    try
    {
      // Create service
      ClientSetup clientSetup = new ClientSetup(System.getProperty("services.namespace"),
                                                System.getProperty("services.namespace.schema"));
      MailingService service = clientSetup.createService(wsdlFile, endpoint);

      // Apply interceptor to add SOAP header
      // username/password/organization are the same as UI login.
      clientSetup.applyInterceptor(service, "admin", "admin", "admin", null, false);
      /*
        For enabling SSO token authentication, set the last argument in the above method to true
        and specify the token in the password (i.e 2nd) argument. E.g.
      clientSetup.applyInterceptor(service, "admin", "<token>", "admin", null, true);
       */


      return service;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.exit(1);
    }

    return null;
  }


  private void getAndPrintData(ObjectId objectId) throws Exception
  {
    if (objectId instanceof DataSourceId)
    {
      new DataSourceSample(service).getAndPrint((DataSourceId)objectId);
    }
    else if (objectId instanceof TargetId)
    {
      new TargetSample(service).getAndPrint((TargetId)objectId);
    }
    else if (objectId instanceof SeedListId)
    {
      new SeedListSample(service).getAndPrint((SeedListId)objectId);
    }
    else if (objectId instanceof SuppressionListId)
    {
      new SuppressionListSample(service).getAndPrint((SuppressionListId)objectId);
    }
    else if (objectId instanceof AttachmentId)
    {
      new AttachmentSample(service).getAndPrint((AttachmentId)objectId);
    }
    else if (objectId instanceof ContentBlockId)
    {
      new ContentBlockSample(service).getAndPrint((ContentBlockId)objectId);
    }
    else if (objectId instanceof RuleId)
    {
      new RuleSample(service).getAndPrint((RuleId)objectId);
    }
    else if (objectId instanceof BaseTemplateId)
    {
      new TemplateSample(service).getAndPrint((BaseTemplateId)objectId);
    }
    else if (objectId instanceof MailingId)
    {
      new MailingSample(service).getAndPrint((MailingId)objectId);
    }
  }

  private static List<ObjectId> getCreatedObjects()
  {
    if(objectIds == null)
    {
      objectIds = new ArrayList<>();
    }
    return objectIds;
  }

  private static List<ObjectId> objectIds;


  /**
   * Get an ObjectId based on its type.
   *
   * @param clazz     the Class whose Id will be returned
   * @return an ObjectId that matches the class
   */
  public static ObjectId getObjectId(Class clazz)
  {
    for (ObjectId objectId : objectIds)
    {
      if (clazz.isInstance(objectId))
      {
        return objectId;
      }
    }

    return null;
  }
}
